#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jun  1 15:11:33 2019

@author: Gabriele Coiana
"""
import numpy as np
from decomp import  read,  util
import sys

# =============================================================================
# Parameters
input_file = sys.argv[1]
Masses = read.read_parameters(input_file)[0]
n_atom_unit_cell = read.read_parameters(input_file)[1]
N1,N2,N3 = read.read_parameters(input_file)[2:5]
kinput_scaled = read.read_parameters(input_file)[5::][0]
file_FC = read.read_parameters(input_file)[6]
file_trajectory = read.read_parameters(input_file)[7]
file_initial_conf = read.read_parameters(input_file)[8]
DT = read.read_parameters(input_file)[9]

tot_atoms_uc = int(np.sum(n_atom_unit_cell)) 
N1N2N3 = N1*N2*N3 # Number of cells
N = N1*N2*N3*tot_atoms_uc    # Number of atoms
cH = 1.066*1e-6 # to [H]
cev = 2.902*1e-05 # to [ev]
kbH = 3.1668085639379003*1e-06# a.u. [H/K]
kbev = 8.617333262*1e-05 # [ev/K]
# =============================================================================

print('\nHello, lets start!\n')
print('Getting input parameters...')
print(' Masses: ', Masses)
print(' Number of atoms per unit cell', n_atom_unit_cell)
print(' Supercell: ', N1, N2, N3)
print(' k path: ', [x.tolist() for x in kinput_scaled])
print(' Extent of timestep [ps]: ', DT*2.418884254*1e-05)
print()


print('Now reading FORCE_CONSTANTS...')
K = read.read_FC(file_FC)
print('Now calculating velocities...')

# =============================================================================
# Supercell and BZ cration
Nqs_path, ks_path, ks_scaled_path, kk, x_labels, Hk = util.get_commensurate_kpath(file_initial_conf, N1,N2,N3, N1,N2,N3, kinput_scaled, np.array([' ' for i in range(len(kinput_scaled))]))
SCell, Ruc, Suc, R0, R0_repeat, S0, masses, masses_uc = read.read_SPOSCAR_and_masses(file_initial_conf, n_atom_unit_cell, Masses, N1, N2, N3)
# =============================================================================

# =============================================================================
# MD positions and velocities
Rt = np.loadtxt(file_trajectory)[:,1:]
Num_timesteps = int(len(Rt[:,0]))
print(' Number of timesteps of simulation: ', Num_timesteps)
tall = np.arange(Num_timesteps)*DT*2.418884254*1e-05 #conversion to picoseconds
dt_ps = tall[1]-tall[0]
Vt = np.diff(Rt,axis=0)/dt_ps*np.sqrt(masses)/np.sqrt(3*(N))
T = np.sum(np.average(Vt**2*cev/kbev, axis=0))
print(' Temperature: ',T, ' K')
# =============================================================================

# =============================================================================
# Global vibrational DOS and C(t)
meta = util.max_freq(dt_ps, Num_timesteps) #you want the max frequency plotted be 25 Thz
ZS, Zqs = np.zeros((meta,1+1+2+Nqs_path)), np.zeros((Nqs_path,meta,tot_atoms_uc*3+1))
Vw, freq = np.fft.fft(Vt, axis=0), np.fft.fftfreq(Num_timesteps-1, dt_ps)
print(' Frequency resolution [Thz]: ', freq[1]-freq[0])
S = np.sum(np.conjugate(Vw)*Vw, axis=1).real*cev/(kbev*T)/Num_timesteps*dt_ps
tot_area = np.trapz(S,freq)
print(' The total DOS is ', tot_area*N)
print(' You are losing ',N - tot_area*N, ' kBT')
ZS[:,0], ZS[:,1] = freq[0:meta], S[0:meta]

Ctot = np.fft.ifft(S)
for t in range(int(len(Ctot)/2)):
    Ctot[t] = Ctot[t]*Num_timesteps/(Num_timesteps-t)
C = Ctot[0:int(len(Ctot)/2)].real
# =============================================================================


quasiparticles_kpoints, quasiparticles_info = np.zeros((Nqs_path, 3)), np.zeros((Nqs_path, 3, tot_atoms_uc*3))
print('\nDone. Performing calculation of renormalised FORCE CONSTANTS...\n')
kpoints_scal, kpoints = util.get_kgrid(N1,N2,N3, Hk)
# you're not using irreducible kpoints anymore, couse it doesn't work for BaTiO3, unless you do adjustments
# irr_kpoints, irr_kpoints_scal, weights, permutations_types = util.get_irr_ks(kpoints, kpoints_scal)
Ds = np.zeros((len(kpoints),tot_atoms_uc*3, tot_atoms_uc*3), dtype=complex)
for i in range(len(kpoints)):
    k = kpoints[i]
    k_scal = kpoints_scal[i]
    # thisk_permutations = permutations_types[i]
    # weight = weights[i]
    print('\t kpoint scaled: ', i, k_scal)
    # print('\t weight: ', weight)
    
# =============================================================================
# harmonic things
    D = util.get_D(k, K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
    eigvals, eigvecs, omegas = util.get_eigvals_eigvec(D)
    eigvecH = np.conjugate(eigvecs.T)
# =============================================================================
    
# =============================================================================
#  MD 
    #Creating the collective variable based on the k point
    Tkt = util.create_Tkt(Num_timesteps-1, tot_atoms_uc, N1N2N3, Vt, R0_repeat, k)    
    Tkw = np.fft.fft(Tkt, axis=0)
    Sq = (np.sum(np.conjugate(Tkw)*Tkw, axis=1)).real*cev/(kbev*T)/Num_timesteps*dt_ps
    area_q = np.trapz(Sq, freq)
    print('\t DOS for this kpoint: ', area_q)
    #Projecting onto eigenvectors
    Qkt = np.dot(eigvecH,Tkt.T).T
    Qkw = np.fft.fft(Qkt, axis=0)
    Sq_proj = (np.conjugate(Qkw)*Qkw).real*cev/(kbev*T)/Num_timesteps*dt_ps
    Params = np.zeros((3,tot_atoms_uc*3))
    for n in range(tot_atoms_uc*3):
        x_data, y_data = freq[0:int(Num_timesteps/2)], Sq_proj[0:int(Num_timesteps/2),n]
        popt, perr = util.fit_to_lorentzian(x_data, y_data, k, n)
        Params[:,n] = popt
        #now you go apply to Voigt equation for the sigmas
        sig = popt[-1]*0.42
        fmax, fl = util.voigt_eq(n, k_scal, x_data, y_data, sig)
        Params[-1,n] = fl
        if(np.any(np.all(np.isclose(k_scal , ks_scaled_path), axis=1))):
            print()
            print('\t\tFitting to Lorentzian, mode '+str(n)+'...')
            print('\t\tResonant frequency omega =',np.round(popt[0],2),' +-',np.round(perr[0],3))
            print('\t\tLinewidth gamma =',np.round(popt[2],2),' +-',np.round(perr[2],3))
            print()
# =============================================================================

# =============================================================================
#  renormalised D
    eigvals_renorm = (Params[0,:]*2*np.pi)**2
    omegas_renorm = np.sqrt(eigvals_renorm)/2/np.pi
    Lamdas_renorm = np.diag(eigvals_renorm)
    D_renorm = np.dot(np.dot(eigvecs,Lamdas_renorm),eigvecH)
    Ds[i,:,:] = D_renorm*np.sqrt(np.outer(masses_uc, masses_uc))
# =============================================================================

# =============================================================================
#  writing to file if in path
    check_is_symm_kpoint = util.check_is_symm(k_scal, ks_scaled_path)
    if(len(check_is_symm_kpoint) > 0):
        # print(check_is_symm_kpoint)
        for l in range(len(check_is_symm_kpoint)):
            kpoint = check_is_symm_kpoint[l]
            index_q = np.argwhere(np.all(np.isclose(kpoint , ks_scaled_path), axis=1)).flatten()
            quasiparticles_kpoints[index_q,:] = kpoint
            quasiparticles_info[index_q,:,:] = Params
            ZS[:,index_q+2] = Sq[0:meta].reshape(meta,1)
            Zqs[index_q,:,0] = Sq[0:meta]
            Zqs[index_q,:,1:] = Sq_proj[0:meta,:]
# =============================================================================
    print()

    
util.save_quasiparticles('Zqs', quasiparticles_kpoints, Zqs)
util.save_quasiparticles('quasiparticles', quasiparticles_kpoints, quasiparticles_info)  
util.save('C_t', np.vstack((tall[0:len(C)],C)).T)
util.save('ZS', ZS)



# SCell_fc, R0_fc, R_fc = util.get_R_FC(N1, N2, N3, 4,4,4,tot_atoms_uc, Suc, SCell) 
K_renorm = util.get_K_renorm_old(tot_atoms_uc,N1,N2,N3, SCell, R0, kpoints, Ds)
# K_renorm = util.get_K_renorm(tot_atoms_uc,N1,N2,N3, SCell, R0, irr_kpoints, permutations_types, Ds)
print('Calculated renormalized FORCE CONSTANTS')
util.save('K_renorm', K_renorm)



# -*- coding: utf-8 -*-
"""
Created on Wed Oct 28 16:01:48 2020

@author: Gabriele Coiana
"""
from scipy.optimize import curve_fit
from scipy.interpolate import interp1d
from scipy import signal
import numpy as np
import os
import warnings
warnings.filterwarnings('ignore') 
from scipy.interpolate import interp1d
import matplotlib as mpl
# mpl.use('pgf')
import matplotlib.pyplot as plt
# from mpl_toolkits.mplot3d import Axes3D
import matplotlib.gridspec as gridspec
from matplotlib import animation
from scipy import interpolate

def plot4_old(kk, frequencies, frequencies_disp, qpoints, x_labels, ZQS):
    """
    old plot 4, with mathematical interpolation
    """
    pgf_with_latex = {                      # setup matplotlib to use latex for output

    "pgf.texsystem": "pdflatex",        # change this if using xetex or lautex

    "text.usetex": True,                # use LaTeX to write all text

    "font.family": "serif",

    "font.serif": [],                   # blank entries should cause plots to inherit fonts from the document

    "font.sans-serif": [],

    "font.monospace": [],

    "axes.labelsize": 16,                # LaTeX default is 10pt font.     "text.fontsize": 10,

    "legend.fontsize": 'x-small',               # Make the legend/label fonts a little smaller

    "xtick.labelsize": 16,

    "ytick.labelsize": 16,

    "figure.figsize": figsize(1.4),       # default fig size of 0.9 textwidth

    "pgf.preamble": [

        r"\usepackage[utf8x]{inputenc}",    # use utf8 fonts becasue your computer can handle it :)

        r"\usepackage[T1]{fontenc}",        # plots will be generated using this preamble

        ]

    }

#    mpl.rcParams.update(pgf_with_latex)
    freq = ZQS[:,0] 
    fig = plt.figure()
    gs = gridspec.GridSpec(1,5, wspace=0)
    ax = fig.add_subplot(gs[0, 0:4])
    plt.xticks(kk, x_labels)
    ax2 = fig.add_subplot(gs[0, 4], sharey=ax)
    n_branches = len(frequencies[0,:,0])
    
    
    
    gamma_pos = []
    for c in range(len(qpoints)):
        qpoint = qpoints[c]
        if(np.allclose(qpoint,[0,0,0])):
            gamma_pos.append(c)
    if(gamma_pos[0] != 0):
        gamma_pos.insert(0,0)
    if(gamma_pos[-1] != len(kk)):
        gamma_pos.insert(len(kk),len(kk))
    
    
    gaussians = np.zeros(len(freq)) 
    gaussians_ft = np.zeros(len(freq))
    for j in range(3):
#        ax.scatter(kk,frequencies_disp[0,j,:], marker='x', color='black', label='0K dispersion')
#        ax.scatter(kk,frequencies[0,j,:], marker='x', color='red')
        for d in range(len(gamma_pos)-1):
            this_gamma = gamma_pos[d]
            next_gamma = gamma_pos[d+1]+1
            kk_div = kk[this_gamma:next_gamma]
            f = interpolate.interp1d(kk_div, frequencies[0,j,this_gamma:next_gamma], kind=3)
            f_disp = interpolate.interp1d(kk_div, frequencies_disp[0,j,this_gamma:next_gamma], kind=3)
            x = np.arange(kk_div.min(), kk_div.max(), 0.01)
            ax.plot(x, f(x), color='red')
            ax.plot(x, f_disp(x), color='black')
            f_gamma = interpolate.interp1d(kk_div, frequencies[1,j,this_gamma:next_gamma], kind=1)
            y1, y2 = f(x)+f_gamma(x)/2, f(x)-f_gamma(x)/2
            ax.fill_between(x, y1, y2, color='red', alpha=0.3)
            for g in range(len(x)):
                mi_disp = f_disp(x)[g]
                mi_ft = f(x)[g]
                
                if(np.isclose(mi_disp,0, rtol=0.1, atol=0.1) or np.isclose(mi_ft,0,rtol=0.1, atol=0.1)):
                    continue

                gauss = util.norm_gaussian(freq, mi_disp, .1)
                gauss_ft = util.norm_gaussian(freq,mi_ft, .1)
                gaussians = gaussians + gauss
                gaussians_ft = gaussians_ft + gauss_ft
    
    for j in range(3,n_branches):
        f = interpolate.interp1d(kk, frequencies[0,j,:], kind=3)
        f_disp = interpolate.interp1d(kk, frequencies_disp[0,j,:], kind=3)
        x = np.arange(kk.min(), kk_div.max(), 0.01)
        graph_ft, = ax.plot(x, f(x), color='red', label='finite T dispersion')
        graph_disp, = ax.plot(x, f_disp(x), color='black', label='0K dispersion')
        f_gamma = interpolate.interp1d(kk, frequencies[1,j,:], kind=1)
        y1, y2 = f(x)+f_gamma(x)/2, f(x)-f_gamma(x)/2
        ax.fill_between(x, y1, y2, color='red', alpha=0.3)
#        graph_disp = ax.scatter(kk,frequencies_disp[0,j,:], marker='x', color='black', label='0K dispersion')
#        graph_ft = ax.scatter(kk,frequencies[0,j,:], marker='x', color='red', label='finite T dispersion')
        
        for g in range(len(x)):
            mi_disp = f_disp(x)[g]
            mi_ft = f(x)[g]
            gauss = util.norm_gaussian(freq, mi_disp, 0.1)
            gauss_ft = util.norm_gaussian(freq,mi_ft, 0.1)
            gaussians = gaussians + gauss
            gaussians_ft = gaussians_ft + gauss_ft
        
    tot_DOS = np.trapz(gaussians, freq)
    tot_DOS_ft = np.trapz(gaussians_ft, freq)
    
    ax.set_ylabel('Frequency [Thz]')
    ax.tick_params(axis='both', which='major', labelsize=15, width=2 , length=6, direction='in', top=True, right=True)
    ax.axhline(0, linestyle='--', color='b')
    
#    print(np.trapz(gaussians/tot_DOS/2,freq))
#    print(np.trapz(np.sum(ZQS[:,1:], axis=1), freq))
    DOS_ft, = ax2.plot(np.sum(ZQS[:,1:], axis=1), freq, c='red', label='finite T DOS')
#    DOS_ft, = ax2.plot(gaussians_ft/tot_DOS_ft/2, freq, c='red', label='finite T DOS')
#    DOS_0K,= ax2.plot(gaussians/tot_DOS/2,freq, c='black', label = '0K DOS')
    ax2.set_xticks([])
    ax2.tick_params(axis='y', which='major', labelsize=0.001, width=2 , length=6, direction='in', top=True, right=True)
    ax2.legend()
    
    plt.rcParams['axes.linewidth'] = 2
    ax.set_xticks(kk, x_labels)
    ax.legend(handles=[graph_disp, graph_ft])
#    ax2.legend()
#    plt.show()
    fig.savefig('plot4.pdf')
    return

def plot_k2_old(f, data, data_projected, indexes,freqs, max_Z, branches, ks_path, title=''):
    pgf_with_latex = {                      # setup matplotlib to use latex for output

    "pgf.texsystem": "pdflatex",        # change this if using xetex or lautex

    "text.usetex": True,                # use LaTeX to write all text

    "font.family": "serif",

    "font.serif": [],                   # blank entries should cause plots to inherit fonts from the document

    "font.sans-serif": [],

    "font.monospace": [],

    "axes.labelsize": 16,                # LaTeX default is 10pt font.     "text.fontsize": 10,

    "legend.fontsize": 12,               # Make the legend/label fonts a little smaller

    "xtick.labelsize": 16,

    "ytick.labelsize": 16,

    "figure.figsize": figsize(1.4),       # default fig size of 0.9 textwidth

    "pgf.preamble": [

        r"\usepackage[utf8x]{inputenc}",    # use utf8 fonts becasue your computer can handle it :)

        r"\usepackage[T1]{fontenc}",        # plots will be generated using this preamble

        ]

    }

    mpl.rcParams.update(pgf_with_latex)
    num_ks = len(data_projected[0,0,:])
    tot_branches = int(len(freqs[0][0,:]))
    plt.ion()
    kk = np.linspace(0,1,num_ks)
    X, Y = np.meshgrid(kk,f)
    
    fig, ax = plt.subplots()
    
    
#    if(len(branches) == tot_branches): #plot whole spectrum
#        if(max_Z > np.max(data) or max_Z < 0.0):
#            max_Z = np.max(data)
#        print('Drawing the whole spectrum...')
#        ax.contourf(X,Y,data,100, vmax=max_Z)
#        print('finished')

    if(max_Z > np.max(data_projected) or max_Z < 0.0):
        max_Z = np.max(data_projected)
    print('Drawing branches ', branches)
    ax.contourf(X,Y,np.sum(data_projected, axis=1),100, vmax=max_Z)
    print('finished')
        
    num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries = util.find_Gammas(ks_path)
    
    
    
    ax2 = plt.twinx()
    ax2.set_ylabel('[Thz]')
    num_dispersions = len(freqs)
    colours = ['black', 'red']
    graphs = [0,0]
    num_ks_commensurate = len(freqs[0][:,0])
    kk_commensurate = np.linspace(0,1,num_ks_commensurate)
    acou_branches = [b for b in branches if b in [0,1,2]]
    optic_branches = [b for b in branches if b not in [0,1,2]]
    for l in range(1):#(num_dispersions):
        for j in acou_branches:
           for m in range(len(indexes_Gammas)):
                this_G = indexes_Gammas_withboundaries[m]
                next_G = indexes_Gammas_withboundaries[m+1]+1
                kk_i = kk_commensurate[this_G:next_G]
                f_i = freqs[l][this_G:next_G,j]
                interp = interp1d(kk_i, f_i, 'cubic')
                x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
                graphs[l] = ax2.plot(x_i,interp(x_i),  c=colours[l], label=title[l])
#           ax2.scatter(kk_commensurate, freqs[l][:,j], c='red')
        for j in optic_branches:
            kk_i = kk_commensurate
            f_i = freqs[l][:,j]
            interp = interp1d(kk_i, f_i, 'cubic')
            x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
            graphs[l] = ax2.plot(x_i,interp(x_i),  c=colours[l], label=title[l])
#            ax2.scatter(kk_commensurate, freqs[l][:,j], c='red')
            

           
    ax2.get_shared_y_axes().join(ax, ax2)
    plt.ylim([f[0], f[-1]])
    plt.xlim([kk[0], kk[-1]])
    
    plt.xticks(kk_commensurate,indexes)
    plt.legend(handles=(graphs[0][0],))#graphs[1][0]
    ax.tick_params(axis='both', which='major', labelsize=15, width=2 , length=6, direction='in', top=True, left=True, bottom=True)
    ax2.tick_params(axis='y', which='major', labelsize=15, width=2 , length=6, direction='in')
    plt.rcParams['axes.linewidth'] = 2
#    plt.show()
    plt.suptitle('Spectrum branches '+str(branches))
    fig.savefig('plot5_'+str(branches)+'.pdf', backend='pgf')
    return 


def interpol_spectrum_old(ks_path, freq, freqs_from_spectrum, eigvecs, modes, ZQS, new_ks_in_between):
    num_ks = len(ks_path)
    num_modes = len(freqs_from_spectrum[0,:])
    num_freqs = len(ZQS[:,0,0])
    kk = np.linspace(0,1,num_ks)
    num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries = find_Gammas(ks_path)
    df = freq[1] - freq[0]
    X, Y = np.meshgrid(kk,freq)
    
#    get_Force_Constants(eigvecs, freqs_from_spectrum.T)
    interp_freq = []
    INTERP = []
    for j in range(len(modes)):
        interp_freq_interm = np.array([])
        for m in range(len(indexes_Gammas)):
            this_G = indexes_Gammas_withboundaries[m]
            next_G = indexes_Gammas_withboundaries[m+1]+1
            kk_i = kk[this_G:next_G]
            f_i = freqs_from_spectrum[this_G:next_G,j]
            interp = interp1d(kk_i, f_i, 'cubic')
#            x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
            INTERP.append(interp)
#            interp_freq_interm = np.concatenate((interp_freq_interm, interp(x_i)))
#        interp_freq_interm = np.array(interp_freq_interm)
#        interp_freq.append(interp_freq_interm)
#    interp_freq = np.array(interp_freq)

    ZQS_interp = np.zeros((num_freqs,num_modes, new_ks_in_between*(num_ks-1)+num_ks))
    
    
    ktot = 0
    for k in range(num_ks-1):
        sector = 0
        for l in indexes_Gammas_withboundaries[1::]:
            if(k >= l):
                sector = sector + 1
        count_k_in_between = 0
        for j in range(num_modes):  
            this_gaussian = ZQS[:,j,k]
            next_gaussian = ZQS[:,j,k+1]
            
#            import matplotlib.pyplot as plt
#            fig,ax = plt.subplots()
#            #to comment
#            popt_this, pcov_this = curve_fit(gaussian, freq, this_gaussian)
#            popt_next, pcov_next = curve_fit(gaussian, freq, next_gaussian)
            
            this_max = freqs_from_spectrum[k,j]
            next_max = freqs_from_spectrum[k+1,j]
            diff_maxima = next_max - this_max
            
            if(diff_maxima >= 0):
                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
                this_gaussian_fit = this_gaussian_interp(freq-diff_maxima)
                next_gaussian_fit = next_gaussian_interp(freq)
                shift_interp = -next_max
            else:
                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
                this_gaussian_fit = this_gaussian_interp(freq)
                next_gaussian_fit = next_gaussian_interp(freq+diff_maxima)
                shift_interp = -this_max
            
#            area_this_gaussian = np.trapz(this_gaussian_fit, freq)
#            area_next_gaussian = np.trapz(next_gaussian_fit, freq)

            this_k = kk[k]
            next_k = kk[k+1]
            interm_k = np.linspace(this_k, next_k, new_ks_in_between+2)[0:-1]

            interp = INTERP[sector+j*num_Gammas]
            intermediate_maxima = interp(interm_k)
            
            count_k_in_between = 0
            for m in range(new_ks_in_between+1):
                mi = intermediate_maxima[m]
                x = (interm_k[m] - interm_k[0])/(next_k-interm_k[0])
#                area_factor = area_this_gaussian+(area_next_gaussian-area_this_gaussian)*x
                interp_gaussian = this_gaussian_fit+(next_gaussian_fit-this_gaussian_fit)*x
                interp_gaussian_interp = interp1d(freq, interp_gaussian,bounds_error=False, fill_value=0)
                interp_gaussian_fit = interp_gaussian_interp(freq-(shift_interp+mi))
                ZQS_interp[:,j,ktot+count_k_in_between] = interp_gaussian_fit
                count_k_in_between = count_k_in_between + 1
                

#                popt = popt_this + (popt_next-popt_this)*x
#                interp_gaussian_fit = gaussian(freq, *popt)#1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2) *area_factor

#                ax.plot(freq, interp_gaussian_fit, label='interp')
#            ax.plot(freq, this_gaussian, label='real this')
#            ax.plot(freq,next_gaussian, label='real next')
##            ax.plot(freq, this_gaussian_fit, label='real this fit')
##            ax.plot(freq,next_gaussian_fit, label='real next fit')
#            ax.legend()
            
        ktot = ktot + count_k_in_between
    ZQS_interp[:,:,-1] = ZQS[:,:,-1]
    return ZQS_interp

def get_nac_wrong(kpoint,Vuc, BORN, eps_inf, masses):
    const = 1.18*1e+07
    num = np.outer(np.dot(kpoint,BORN.T) , np.dot(BORN,kpoint))
    den = np.dot(kpoint.T,np.dot(eps_inf,kpoint))
    Cna_k = 4*np.pi/Vuc * num/den #* const
    
#                    thisq_eigvec = eigvecs_phonopy[j]
#                    M = masses_for_animation[0]*masses_for_animation[3]/(masses_for_animation[0]+masses_for_animation[3])
#                    omega_l = np.sqrt(freqs_from_disp[0][5]**2 + 4*np.pi*BORN[0,0]/(M*eps_inf[0,0]*Vuc))   
#                    nac_q = np.zeros((tot_atoms_uc, tot_atoms_uc, 3, 3), dtype='double', order='C')
#                    born = BORN.reshape((2,3,3))
#                    A = np.dot(kpoint, born)
#                    CC = np.zeros((6,6))
#                    for i in range(tot_atoms_uc):
#                        for j in range(tot_atoms_uc):
#                            nac_q[i, j] = np.outer(A[i], A[j])
#                            CC[i*3:(i+1)*3, j*3:(j+1)*3] = 4*np.pi/Vuc *nac_q[i,j] /den
##                            print(i,j, 4*np.pi/Vuc *nac_q[i,j])
#                    AB = CC
#                    eigvna, eigvecna = np.linalg.eigh(AB)
#                    print(np.sqrt(eigvna))              
#                    lamdas = np.diag(freqs_from_disp[0]**2)
#                    Dq = np.dot(thisq_eigvec, np.dot(lamdas, np.linalg.inv(thisq_eigvec)))
    M1, M2 = np.meshgrid(np.sqrt(masses), np.sqrt(masses))
    Mij = np.multiply(M1,M2)
    D = 1/Mij*Cna_k*const/1000 #+Dq
    eigvals, eigvecs = np.linalg.eigh(D)
    omegas_nac = np.sqrt(eigvals)
    return omegas_nac, D



#def get_real_branches(freqs):
#    num_ks = len(freqs[0,:])
#    num_branches = len(freqs[:,0])
#    indexes0 = np.arange(num_branches)
#    freqs_until_now = freqs[:,0:2]
#    
#    
#    for k in range(1,8):
#        deriv = np.diff(freqs[:,k-1:k+2], axis=1)
#        next_freqs = freqs[:,k+1]
#        this_deriv = deriv[:,1]
#        prev_deriv = deriv[:,0]
#        actual_diff = this_deriv-prev_deriv
#        print(this_deriv[2])
#        print(prev_deriv[2])
#        
#        sopra, sotto = np.ones(num_branches)*100, np.ones(num_branches)*100
#        two_sopra, two_sotto = np.ones(num_branches)*100, np.ones(num_branches)*100
#        
#        for j in range(num_branches-1):
#            down_freq = freqs[j-1,k+1]
#            up_freq = freqs[j+1,k+1]
#            diff_down = down_freq - freqs[j,k] 
#            diff_up = up_freq - freqs[j,k] 
#            sopra[j] = diff_up
#            sotto[j+1] = diff_down
#            
#        for j,l in zip(range(num_branches-2),range(2,num_branches)):
#            two_down_freq = freqs[l-2,k+1]
#            two_up_freq = freqs[j+2,k+1]
#            two_diff_down = two_down_freq - freqs[j,k] 
#            two_diff_up = two_up_freq - freqs[j,k] 
#            two_sopra[j] = two_diff_up
#            two_sotto[j+1] = two_diff_down
#
#        sopra, sotto = np.array(sopra), np.array(sotto)
#        diff_deriv_up = sopra - prev_deriv
#        diff_deriv_down = sotto - prev_deriv
#        gain_up = np.abs(diff_deriv_up)-np.abs(actual_diff)
#        gain_down = np.abs(diff_deriv_down)-np.abs(actual_diff)
#        
#        two_sopra, two_sotto = np.array(two_sopra), np.array(two_sotto)
#        two_diff_deriv_up = two_sopra - prev_deriv
#        two_diff_deriv_down = two_sotto - prev_deriv
#        two_gain_up = np.abs(two_diff_deriv_up)-np.abs(actual_diff)
#        two_gain_down = np.abs(two_diff_deriv_down)-np.abs(actual_diff)
#        
#        print(k)
#        print(np.shape(next_freqs))
#        print(np.round(np.hstack((freqs_until_now[:,-2::],next_freqs.reshape(6,1))),4))
#        print(np.round(next_freqs,4))
##        print()
##        print(np.round(diff_deriv_down,2))
#        print('gaindown',np.round(gain_down,2))
#        print('gainup',np.round(gain_up,2))
#        print('actualdiff', np.round(actual_diff,2))
#        print()
#        for j in range(num_branches-1):
#
#            if(gain_down[j+1] <0 and gain_up[j] <0):
#                print('aaa',j)
#                temp = next_freqs[j]
#                next_freqs[j] = next_freqs[j+1]
#                next_freqs[j+1] = temp
#               
##            elif(two_gain_up[j]<0 and two_gain_down[j+2]<0 and np.abs(next_freqs[j]-next_freqs[j+1])<0.05):
##                print('aaa2',j)
##                temp = next_freqs[j]
##                next_freqs[j] = next_freqs[j+2]
##                next_freqs[j+2] = temp
#        print(next_freqs)
#        freqs_sorted = np.hstack((freqs_until_now,next_freqs.reshape(num_branches,1)))
#        freqs_until_now = freqs_sorted
#        
#        print()
#    return freqs_sorted

def interpol_spectrum_old(ks_path, freq, freqs_from_spectrum, eigvecs, modes, ZQS, new_ks_in_between):
    num_ks = len(ks_path)
    num_modes = len(freqs_from_spectrum[0,:])
    num_freqs = len(ZQS[:,0,0])
    kk = np.linspace(0,1,num_ks)
    num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries = find_Gammas(ks_path)
    df = freq[1] - freq[0]
    X, Y = np.meshgrid(kk,freq)
    
#    get_Force_Constants(eigvecs, freqs_from_spectrum.T)
    interp_freq = []
    INTERP = []
    for j in range(len(modes)):
        interp_freq_interm = np.array([])
        for m in range(len(indexes_Gammas)):
            this_G = indexes_Gammas_withboundaries[m]
            next_G = indexes_Gammas_withboundaries[m+1]+1
            kk_i = kk[this_G:next_G]
            f_i = freqs_from_spectrum[this_G:next_G,j]
            interp = interp1d(kk_i, f_i, 'cubic')
#            x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
            INTERP.append(interp)
#            interp_freq_interm = np.concatenate((interp_freq_interm, interp(x_i)))
#        interp_freq_interm = np.array(interp_freq_interm)
#        interp_freq.append(interp_freq_interm)
#    interp_freq = np.array(interp_freq)

    ZQS_interp = np.zeros((num_freqs,num_modes, new_ks_in_between*(num_ks-1)+num_ks))
    
    
    ktot = 0
    for k in range(num_ks-1):
        sector = 0
        for l in indexes_Gammas_withboundaries[1::]:
            if(k >= l):
                sector = sector + 1
        count_k_in_between = 0
        for j in range(num_modes):  
            this_gaussian = ZQS[:,j,k]
            next_gaussian = ZQS[:,j,k+1]
            
#            import matplotlib.pyplot as plt
#            fig,ax = plt.subplots()
#            #to comment
#            popt_this, pcov_this = curve_fit(gaussian, freq, this_gaussian)
#            popt_next, pcov_next = curve_fit(gaussian, freq, next_gaussian)
            
            this_max = freqs_from_spectrum[k,j]
            next_max = freqs_from_spectrum[k+1,j]
            diff_maxima = next_max - this_max
            
            if(diff_maxima >= 0):
                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
                this_gaussian_fit = this_gaussian_interp(freq-diff_maxima)
                next_gaussian_fit = next_gaussian_interp(freq)
                shift_interp = -next_max
            else:
                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
                this_gaussian_fit = this_gaussian_interp(freq)
                next_gaussian_fit = next_gaussian_interp(freq+diff_maxima)
                shift_interp = -this_max
            
#            area_this_gaussian = np.trapz(this_gaussian_fit, freq)
#            area_next_gaussian = np.trapz(next_gaussian_fit, freq)

            this_k = kk[k]
            next_k = kk[k+1]
            interm_k = np.linspace(this_k, next_k, new_ks_in_between+2)[0:-1]

            interp = INTERP[sector+j*num_Gammas]
            intermediate_maxima = interp(interm_k)
            
            count_k_in_between = 0
            for m in range(new_ks_in_between+1):
                mi = intermediate_maxima[m]
                x = (interm_k[m] - interm_k[0])/(next_k-interm_k[0])
#                area_factor = area_this_gaussian+(area_next_gaussian-area_this_gaussian)*x
                interp_gaussian = this_gaussian_fit+(next_gaussian_fit-this_gaussian_fit)*x
                interp_gaussian_interp = interp1d(freq, interp_gaussian,bounds_error=False, fill_value=0)
                interp_gaussian_fit = interp_gaussian_interp(freq-(shift_interp+mi))
                ZQS_interp[:,j,ktot+count_k_in_between] = interp_gaussian_fit
                count_k_in_between = count_k_in_between + 1
                

#                popt = popt_this + (popt_next-popt_this)*x
#                interp_gaussian_fit = gaussian(freq, *popt)#1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2) *area_factor

#                ax.plot(freq, interp_gaussian_fit, label='interp')
#            ax.plot(freq, this_gaussian, label='real this')
#            ax.plot(freq,next_gaussian, label='real next')
##            ax.plot(freq, this_gaussian_fit, label='real this fit')
##            ax.plot(freq,next_gaussian_fit, label='real next fit')
#            ax.legend()
            
        ktot = ktot + count_k_in_between
    ZQS_interp[:,:,-1] = ZQS[:,:,-1]
    return ZQS_interp


def plot6_old(x,y,ytot, k,eigvec,freq,n,Ruc,eigname,masses, max_Z):
    mpl.rcParams.update(pgf_with_latex)
    
    fig,ax = plt.subplots()
#    graph2, = ax.plot(x,ytot,'--',label='qpoint DOS', color='orangered')
    graph1, = ax.plot(x,y,label='mode-projected DOS', color='blue')
    ax.set_xlim(left=-1,right=30)
    ax.set_xlabel('Frequency [Thz]')
    plt.title('Spectrum of mode n. '+str(n)+' kpoint '+str(np.round(k,2)))
    
#    ax1 = ax.twiny()
#    ax1.set_xlabel('Frequency [cm$^{-1}$]')
#    ax1.plot(x*33.35641,-np.ones(len(x)),c='white',linewidth=0,label='')
#    ax1.set_xlim(-1*33.35641,30*33.35641)

    graph3, = ax.plot(np.repeat(freq,100),np.linspace(0,y.max(),100),':' ,c='r', label='freq from dispersion')
    plt.legend(handles=[graph1,  graph3], loc=2)
    
    
    ax2 = fig.add_axes([0.5, .5, .4, .4],projection='3d')
#    ax2.set_title(str(eigname)+'\nfrequency: '+str(freq))
    ax2.set_xlabel('x'), ax2.set_ylabel('y'), ax2.set_zlabel('z')
    ampl = 10
    eigvec = eigvec/np.sqrt(masses)*ampl

    print('# =============================================================================')
    print('Kpoint '+str(np.round(k,2))+' Mode '+str(n)+'; frequency: ', freq, '\n')
    print('Eigenvector components:')
    print(np.round(np.reshape(np.real(eigvec),np.shape(Ruc)),2))
    print()
    print('Imaginary part:')
    print(np.round(np.reshape(np.imag(eigvec),np.shape(Ruc)),2))
    print()
    ani = plot_eigvec_noani(Ruc, 1,k,eigvec,freq,ax2,fig, masses)
    
    if(max_Z > np.max(ytot.max()) or max_Z < 0.0):
        a=1
    else:
        ax.set_ylim(bottom=-max_Z/10,top=max_Z)
        
    plt.show()
    plt.tight_layout()
    fig.savefig('plot6_'+str(np.round(k))+'_mode_'+str(n)+'.pdf')
    return ani


def read_path(kinput_scaled, Nqpoints, labels, Hk):
    Nq_input = len(kinput_scaled)
    ks = []
    x_labels = []
    distances = []
    previous_k = np.dot(Hk, kinput_scaled[0])
    distance = 0.0
    for i in range(Nq_input-1):
        this_k_input = kinput_scaled[i]
        next_k_input = kinput_scaled[i+1]
        ks.append(this_k_input)
        x_labels.append(labels[i])
        direction_input = (next_k_input - this_k_input)/np.linalg.norm(next_k_input - this_k_input)
        accepted_kinputs = []
        for j in range(Nqpoints):
            to_skip = (4)*j
            this_k = np.genfromtxt('quasiparticles',skip_header=to_skip+0, max_rows=1)
            print(this_k)
            direction = (this_k - this_k_input)/np.linalg.norm(this_k - this_k_input)
            print(direction,direction_input)
            if(np.allclose(direction, direction_input) and not np.allclose(this_k, next_k_input)):
                print('aaa')
                ks.append(this_k)
                distance = distance + np.linalg.norm(np.dot(Hk,this_k) - previous_k)
                distances.append(distance)
                previous_k = np.dot(Hk,this_k)
                x_labels.append(' ')
            elif(np.allclose(this_k, this_k_input) and not this_k.tolist() in accepted_kinputs):
                accepted_kinputs.append(this_k_input.tolist())
                this_q = np.dot(Hk, this_k_input)
                distance = distance + np.linalg.norm(this_q - previous_k)
                distances.append(distance)  
                previous_k = this_q
    
    ks.append(kinput_scaled[-1])
    x_labels.append(labels[-1])
    distance = distance + np.linalg.norm(np.dot(Hk,kinput_scaled[-1])-previous_k)
    distances.append(distance)
    distances = np.array(distances)
    return ks, x_labels, distances


def plot(x,y,title):
    fig,ax = plt.subplots()
    ax.plot(x,y)
    ax.set_xlabel('Frequency [Thz]')
    ax.set_title(title)
    plt.show()
    return


def from_band(ks):
    Points = {'$\Gamma$':[0,0,0],'X':[0.5,0,0],'M':[0.5,0.5,0],'R':[0.5,0.5,0.5],'boh':[0,0.5,0], 'Z':[0,0,0.5]}
    x_labels = []
    for kpoint in ks:
        a = 0
        for element in Points.items():
            a = a + 1
            if (np.array_equal(kpoint , element[1])):
                x_labels.append(element[0])
                break
            if (a==len(Points.items())):
                x_labels.append(' ')
    return x_labels


def plot_k(f, data, data_projected, indexes,freqs, max_Z, branches,  freq_res, title=''):
    """
    Plotting the dispersion from MD and from phonon theory.
    
    Variables:
        f = array of the frequencies [Thz] 
        data = matrix with spectra from all k points: shape = (timesteps,N)
        indexes = list of all kpoints
        freqs = frequencies from phonopy
    """
    
    pgf_with_latex = {                      # setup matplotlib to use latex for output

    "pgf.texsystem": "pdflatex",        # change this if using xetex or lautex

    "text.usetex": True,                # use LaTeX to write all text

    "font.family": "serif",

    "font.serif": [],                   # blank entries should cause plots to inherit fonts from the document

    "font.sans-serif": [],

    "font.monospace": [],

    "axes.labelsize": 16,                # LaTeX default is 10pt font.     "text.fontsize": 10,

    "legend.fontsize": 12,               # Make the legend/label fonts a little smaller

    "xtick.labelsize": 16,

    "ytick.labelsize": 16,

    "figure.figsize": figsize(1.4),       # default fig size of 0.9 textwidth

    "pgf.preamble": [

        r"\usepackage[utf8x]{inputenc}",    # use utf8 fonts becasue your computer can handle it :)

        r"\usepackage[T1]{fontenc}",        # plots will be generated using this preamble

        ]

    }

    mpl.rcParams.update(pgf_with_latex)

    
    fig,ax = plt.subplots()
    plt.suptitle('Spectrum via MD and phonon theory')
    #ax.set_title(title)
    ax.set_ylabel('[Thz]')
    N = len(data[0,:])
    offset = np.zeros(N) + f[0]
    df = f[1]-f[0]
    molt = int(freq_res/df)
    tot_branches = int(len(freqs[0][0,:]))
    
    kk = np.linspace(0,1,N)
    dk = kk[1] - kk[0]
    
    

#    indexes = from_band(ks)
    
    #norm = [plt.cm.colors.Normalize(vmax=abs(data[0::1,l]).max(), vmin=-abs(data[0::1,l]).min()) for l in range(len(data[0,:]))]
    if(len(branches) == tot_branches): #plot whole spectrum
        print('Drawing the whole spectrum...')
        if(max_Z > np.max(data) or max_Z < 0.0):
            max_Z = np.max(data[::molt,:])
        norm = plt.cm.colors.Normalize(vmax=max_Z, vmin=0)
        for j in range(0,len(data[:,0]),molt):  
            # this is the whole spectrum
            ax.bar(kk, np.repeat(df*molt,N), bottom=offset-df/2,  color=plt.cm.Blues(norm(data[j,:])), tick_label=indexes, align='center', width=dk, alpha=None)
            offset = offset + df*molt
            
    else:   #print only those branches 
        if(max_Z > np.max(data_projected) or max_Z < 0.0):
             max_Z = np.max(data_projected)
        norm = plt.cm.colors.Normalize(vmax=max_Z, vmin=0)
        cmps = [plt.cm.Purples, plt.cm.Reds, plt.cm.Greens, plt.cm.Greys, plt.cm.Oranges]
        print('Drawing spectrum branches ', branches)
        for j in range(0,len(data[0::1,0])):
            for mm in range(len(data_projected[0,:,0])):
                if(np.max(data_projected[j,mm,:]) == np.max(data_projected[:,mm,:])): #this is just for the legend
                    ax.bar(kk, np.repeat(df,N), bottom=offset-df/2,  color=cmps[mm]((data_projected[j,mm,:])), tick_label=indexes, align='center', width=dk, alpha=.5)
                    kpoint_of_max = np.argwhere(data_projected[j,mm,:] == np.max(data_projected[j,mm,:]))[0,0]
                    bar = ax.bar(kk[kpoint_of_max], df, bottom=offset[0]-df/2,  color=cmps[mm]((data_projected[j,mm,kpoint_of_max])),  align='center', width=dk, alpha=.5, label='mode '+str(branches[0]+mm))
                else:
#                    print(j, (data_projected[j,mm,:]))
                    ax.bar(kk, np.repeat(df,N), bottom=offset-df/2,  color=cmps[mm]((data_projected[j,mm,:])), tick_label=indexes, align='center', width=dk, alpha=.5)

            offset = offset + df
   
    
    ax2 = plt.twinx()
    ax2.set_ylabel('[Thz]')
    num_dispersions = len(freqs)
    colours = ['black', 'red']
    for l in range(num_dispersions):
        for j in range(tot_branches):
            ax2.scatter(kk,freqs[l][:,j], marker='x', c=colours[l])
        ax2.scatter(kk,freqs[l][:,tot_branches-1],  marker='x',  c=colours[l], label=title[l])
    ax2.get_shared_y_axes().join(ax, ax2)
    
    
    handles, labels = ax.get_legend_handles_labels()

    plt.ylim([f[0], f[-1]])
    plt.legend(handles=handles, labels=labels)
    ax.tick_params(axis='both', which='major', labelsize=15, width=2 , length=6, direction='in', top=True, left=True, bottom=True)
    ax2.tick_params(axis='y', which='major', labelsize=15, width=2 , length=6, direction='in')
#    plt.show()
    fig.savefig('spectrum_branches_'+str(branches)+'.pdf')
    fig.savefig('spectrum_branches_'+str(branches)+'.png')
    return

def plot_k2_old(f, data, data_projected, indexes,freqs, max_Z, branches, ks_path, title=''):
    pgf_with_latex = {                      # setup matplotlib to use latex for output

    "pgf.texsystem": "pdflatex",        # change this if using xetex or lautex

    "text.usetex": True,                # use LaTeX to write all text

    "font.family": "serif",

    "font.serif": [],                   # blank entries should cause plots to inherit fonts from the document

    "font.sans-serif": [],

    "font.monospace": [],

    "axes.labelsize": 16,                # LaTeX default is 10pt font.     "text.fontsize": 10,

    "legend.fontsize": 12,               # Make the legend/label fonts a little smaller

    "xtick.labelsize": 16,

    "ytick.labelsize": 16,

    "figure.figsize": figsize(1.4),       # default fig size of 0.9 textwidth

    "pgf.preamble": [

        r"\usepackage[utf8x]{inputenc}",    # use utf8 fonts becasue your computer can handle it :)

        r"\usepackage[T1]{fontenc}",        # plots will be generated using this preamble

        ]

    }

    mpl.rcParams.update(pgf_with_latex)
    num_ks = len(data_projected[0,0,:])
    tot_branches = int(len(freqs[0][0,:]))
    plt.ion()
    kk = np.linspace(0,1,num_ks)
    X, Y = np.meshgrid(kk,f)
    
    fig, ax = plt.subplots()
    
    
#    if(len(branches) == tot_branches): #plot whole spectrum
#        if(max_Z > np.max(data) or max_Z < 0.0):
#            max_Z = np.max(data)
#        print('Drawing the whole spectrum...')
#        ax.contourf(X,Y,data,100, vmax=max_Z)
#        print('finished')

    if(max_Z > np.max(data_projected) or max_Z < 0.0):
        max_Z = np.max(data_projected)
    print('Drawing branches ', branches)
    ax.contourf(X,Y,np.sum(data_projected, axis=1),100, vmax=max_Z)
    print('finished')
        
    num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries = util.find_Gammas(ks_path)
    
    
    
#     ax2 = plt.twinx()
#     ax2.set_ylabel('[Thz]')
#     num_dispersions = len(freqs)
#     colours = ['black', 'red']
#     graphs = [0,0]
#     num_ks_commensurate = len(freqs[0][:,0])
#     kk_commensurate = np.linspace(0,1,num_ks_commensurate)
#     acou_branches = [b for b in branches if b in [0,1,2]]
#     optic_branches = [b for b in branches if b not in [0,1,2]]
#     for l in range(1):#(num_dispersions):
#         for j in acou_branches:
#            for m in range(len(indexes_Gammas)):
#                 this_G = indexes_Gammas_withboundaries[m]
#                 next_G = indexes_Gammas_withboundaries[m+1]+1
#                 kk_i = kk_commensurate[this_G:next_G]
#                 f_i = freqs[l][this_G:next_G,j]
#                 interp = interp1d(kk_i, f_i, 'cubic')
#                 x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
#                 graphs[l] = ax2.plot(x_i,interp(x_i),  c=colours[l], label=title[l])
# #           ax2.scatter(kk_commensurate, freqs[l][:,j], c='red')
#         for j in optic_branches:
#             kk_i = kk_commensurate
#             f_i = freqs[l][:,j]
#             interp = interp1d(kk_i, f_i, 'cubic')
#             x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
#             graphs[l] = ax2.plot(x_i,interp(x_i),  c=colours[l], label=title[l])
# #            ax2.scatter(kk_commensurate, freqs[l][:,j], c='red')
            

           
#     ax2.get_shared_y_axes().join(ax, ax2)
    plt.ylim([f[0], f[-1]])
    plt.xlim([kk[0], kk[-1]])
    
    # plt.xticks(kk_commensurate,indexes)
    # plt.legend(handles=(graphs[0][0],))#graphs[1][0]
    ax.tick_params(axis='both', which='major', labelsize=15, width=2 , length=6, direction='in', top=True, left=True, bottom=True)
    ax2.tick_params(axis='y', which='major', labelsize=15, width=2 , length=6, direction='in')
    plt.rcParams['axes.linewidth'] = 2
#    plt.show()
    plt.suptitle('Spectrum branches '+str(branches))
    fig.savefig('plot5_'+str(branches)+'.pdf', backend='pgf')
    return 


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 25 14:33:25 2019

@author: gc4217
"""
from scipy.optimize import curve_fit
from scipy.interpolate import interp1d
from scipy import signal
import numpy as np
import os


def get_commensurate_kpath(file, N1,N2,N3, N1_FC,N2_FC,N3_FC, kinput_scaled, labels):
    # =============================================================================
    #   Brilloin zone 
    conv = np.genfromtxt(file, skip_header=1, max_rows=1)
    h = np.genfromtxt(file,skip_header=2, max_rows=3)
    S = np.genfromtxt(file, skip_header=8)
    
    #your default units are Bohrs, but phonopy's are A
    if (conv==1.0 or conv==1):
        conv_factor = 1.88973
    else:
        conv_factor = 1
        
    f1, f2, f3 = N1_FC/N1, N2_FC/N2, N3_FC/N3
    a1,a2,a3 = h[0]*conv_factor/N1,h[1]*conv_factor/N2,h[2]*conv_factor/N3
    c = 2*np.pi
    
    V = np.dot(a1,np.cross(a2,a3))
    b1 = c*np.cross(a2,a3)/V
    b2 = c*np.cross(a3,a1)/V
    b3 = c*np.cross(a1,a2)/V
    Hk = np.vstack((b1,b2,b3))
    
    b1_mod = np.linalg.norm(b1)
    b2_mod = np.linalg.norm(b2)
    b3_mod = np.linalg.norm(b3)
    b_mod = [b1_mod,b2_mod,b3_mod]
    kinput_real = np.multiply(b_mod,np.array(kinput_scaled))
    #Compute all possible k points
    x1 = np.arange(0,N1_FC+1)/N1_FC
    x2 = np.arange(0,N2_FC+1)/N2_FC
    x3 = np.arange(0,N3_FC+1)/N3_FC
    comb = np.array(np.meshgrid(x1,x2,x3)).T.reshape(-1,3)
    #a1 = np.multiply(comb,b1)
    #a2 = np.multiply(comb,b2)
    #a3 = np.multiply(comb,b3)
    #kpoints = a1+a2+a3
    allkpoints_scaled = comb
    allkpoints = np.multiply(b_mod,comb)
    # =============================================================================
    
    
    # =============================================================================
    # Compute directions <x y z>
    directions_scaled = []
    directions = []
    for i in range(0,len(kinput_scaled)-1):
        if(kinput_scaled[i] not in allkpoints_scaled):
            print('The k point '+str(kinput_scaled[i])+' provided is not compatible with the supercell. \nTry again :)')
        num = 1/np.max(np.abs(kinput_scaled[i]-kinput_scaled[i+1]))
        mod = np.linalg.norm(kinput_scaled[i]-kinput_scaled[i+1])
        directions_scaled.append((kinput_scaled[i]-kinput_scaled[i+1])*num)
        directions.append((kinput_scaled[i]-kinput_scaled[i+1])/mod)
    # =============================================================================
    
    # =============================================================================
    # Remove kpoints already input
    indexes = []
    for i in range(len(allkpoints_scaled)):
        for kpoint in kinput_scaled:
            if(np.array_equal(allkpoints_scaled[i],kpoint)):
                indexes.append(i)
    allkpoints_scaled_no = np.delete(allkpoints_scaled,indexes,axis=0)
    # =============================================================================
    
    # =============================================================================
    # Look for other k points in the same path
    kdef = kinput_real
    kdef_scaled = kinput_scaled
    x_labels = labels
    count = 0
    for index in range(0,len(directions)): #iterating over directions
        count = count+1
        direction = directions[index]
        
        if (all(direction <= np.array([0,0,0]))):
            for kpoint in allkpoints_scaled_no:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
#                    kdef = np.insert(kdef,count,np.dot(h,kpoint),axis=0)   
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    x_labels = np.insert(x_labels,count,' ')
                    count = count+1 
        else:
            for kpoint in allkpoints_scaled_no[::-1]:
                diff = kinput_scaled[index]-kpoint
                mod = np.linalg.norm(diff)
                actual_direction = diff/mod
                if(np.array_equal(np.round(actual_direction,9),np.round(direction,9)) and all(np.abs(diff)<=np.abs(kinput_scaled[index+1]-kinput_scaled[index]))):
#                    kdef = np.insert(kdef,count,np.dot(h,kpoint),axis=0)
                    kdef_scaled = np.insert(kdef_scaled,count,kpoint,axis=0)
                    x_labels = np.insert(x_labels,count,' ')
                    count = count+1 

    # =============================================================================
    
    kdef_scaled = np.array(kdef_scaled)
    kdef = np.dot(Hk,kdef_scaled.T).T
    Nqpoints = len(kdef)
    
    # =============================================================================
    #  fake k points only for plotting, projection
    directionsdef = []
    for i in range(0,len(kdef_scaled)-1):
        directionsdef.append((kdef[i]-kdef[i+1]))
    kk = np.zeros(len(kdef_scaled))
    for i in range(1,len(kk)):
        kk[i] = kk[i-1]+(np.linalg.norm(directionsdef[i-1]))
    # =============================================================================
    
    return Nqpoints, kdef, kdef_scaled, kk, x_labels, Hk

def get_kpath_many(kinput, Hk, npoints=20):
    ks_scal = np.array([0,0,0])
    distances, dist = [], 0
    for i in range(len(kinput)-1):
        this_k = kinput[i]
        next_k = kinput[i+1]
        diff = next_k - this_k
        increment = diff/npoints
        kpoints_in_between = np.outer(increment, np.arange(npoints+1)).T + this_k
        distance = np.linalg.norm(np.dot(Hk,(kpoints_in_between-this_k).T).T, axis=1) + dist
        dist = distance[-1]
        distances.append(distance[0:-1])
        ks_scal = np.vstack((ks_scal,kpoints_in_between[:-1,:]))
        
        
    ks_scal = np.vstack((ks_scal, kinput[-1]))
    ks_scal = ks_scal[1::,:]
    
    distances = np.array(distances).flatten()
    distances = np.concatenate((distances, [distance[-1]]))

    
    ks = np.dot(Hk, ks_scal.T).T
    return ks_scal, ks, distances

def get_kgrid(N1k, N2k, N3k, Hk):
    x,y,z = np.linspace(0,1,N1k+1)[0:-1], np.linspace(0,1,N2k+1)[0:-1], np.linspace(0,1,N3k+1)[0:-1]
    kpoints_scal = np.array(np.meshgrid(x,y,z)).T.reshape(-1,3)
    kpoints = np.dot(Hk,kpoints_scal.T).T
    return kpoints_scal, kpoints

def check_is_symm(k_scal, ks_scal_path):
    permutations = np.array([[[0,1,0],[1,0,0],[0,0,1]], 
                             [[0,0,1],[0,1,0],[1,0,0]], 
                             [[1,0,0],[0,0,1],[0,1,0]],
                             [[0,1,0],[0,0,1],[1,0,0]],
                             [[0,0,1],[1,0,0],[0,1,0]]])
    accepted_ks = []

    #first check: is this irr k within the k path input?
    for j in range(len(permutations)):
        matrix = permutations[j]
        k_permutated = np.dot(matrix, k_scal)
        if (np.any(np.all(np.isclose(k_permutated-ks_scal_path,0), axis=1)) and (k_permutated.tolist() not in accepted_ks) ):
            accepted_ks.append(k_permutated.tolist())
            
    #second check: is this irr k in the k path outside the 1st BZ?
    for s in range(1,3): #3 è un numero a caso, se l'utente va oltre 3 BZ di distanza è proprio un coglione!
        shift = np.eye(3)*s
        ks_outside_BZ = shift + k_scal
        for k in range(3):
            k_outside_BZ = ks_outside_BZ[k,:]
            if (np.any(np.all(np.isclose(k_outside_BZ-ks_scal_path,0), axis=1))):
                accepted_ks.append(k_outside_BZ)
            
    accepted_ks = np.array(accepted_ks)  
    return accepted_ks

def get_irr_ks(ks, ks_scaled):
    Nks = len(ks)
    permutations = np.array([[[0,1,0],[1,0,0],[0,0,1]], 
                             [[0,0,1],[0,1,0],[1,0,0]], 
                             [[1,0,0],[0,0,1],[0,1,0]],
                             [[0,1,0],[0,0,1],[1,0,0]],
                             [[0,0,1],[1,0,0],[0,1,0]]])
    
    accepted_same_ks_overall, index_irr_ks, weights, permutations_types = np.array([]), [], [], []
    for i in range(Nks):
        if(np.isin(i, accepted_same_ks_overall)):
            # print('non irr k') #found non irreducible kpoint
            continue
        this_k = ks[i]
        # print(this_k)
        same_ks_thisk = []
        for j in range(len(permutations)):
            matrix = permutations[j]
            exchanged_rows = np.dot(matrix, ks.T).T
            diff = exchanged_rows-this_k
            w = np.all(np.isclose(diff,0), axis=1)# * 1
            same_ks_index = np.argwhere(w==True).flatten()
            same_ks_thisk.append(same_ks_index.tolist())

        same_ks_thisk = (np.array(same_ks_thisk).flatten())
        # print(same_ks_thisk)
        indexes_without_thisk = np.argwhere(same_ks_thisk != i).flatten()
        same_ks_thisk = same_ks_thisk[indexes_without_thisk]
        permutations_thisk = np.arange(5, dtype=int)[indexes_without_thisk]
        accepted_same_ks_thisk, indexes = np.unique(same_ks_thisk, return_index=True)
        
        # print(accepted_same_ks_thisk)
        accepted_permutations = permutations_thisk[indexes]
        
        
        # print(ks[accepted_same_ks_thisk])
        # print(accepted_permutations)
        # print()
        
        weights.append(1 + len(accepted_same_ks_thisk))
        index_irr_ks.append(i)
        permutations_types.append(accepted_permutations.tolist())
        accepted_same_ks_overall = np.concatenate((accepted_same_ks_overall,accepted_same_ks_thisk))

    
    irr_ks_scal = ks_scaled[index_irr_ks,:]
    irr_ks = ks[index_irr_ks,:]
    
    return irr_ks, irr_ks_scal, np.array(weights), permutations_types


def get_eigvals_eigvec(D):
    eigvals, eigvecs = np.linalg.eigh(D)
    idx = np.argsort(eigvals)
    eigvals, eigvecs = eigvals[idx], eigvecs[:,idx]
    omegas = np.sqrt(eigvals, dtype=complex)/(2*np.pi)
    omegas = omegas.real - omegas.imag
    return eigvals, eigvecs, omegas

def get_eigvals(D):
    eigvals = np.linalg.eigvalsh(D)
    idx = np.argsort(eigvals)
    eigvals = eigvals[idx]
    omegas = np.sqrt(eigvals, dtype=complex)/(2*np.pi)
    omegas = omegas.real - omegas.imag
    return eigvals, omegas
    

def repeat_masses(Masses, n_atom_conventional_cell, n_atom_primitive_cell, N1, N2, N3):
    repeated_masses = np.array([])
    repeated_masses_for_ani = np.array([])
    for i in range(len(Masses)):
        mass = Masses[i]

        n = n_atom_conventional_cell[i]
        nprim = n_atom_primitive_cell[i]
        
        m = np.repeat(mass, N1*N2*N3*3*n)
        m_ani = np.repeat(mass,nprim*3)
        
        repeated_masses = np.concatenate((repeated_masses,m))
        repeated_masses_for_ani = np.concatenate((repeated_masses_for_ani,m_ani))
        
    masses = np.array(repeated_masses).flatten()
    masses_for_animation = np.array(repeated_masses_for_ani).flatten()
    
    return masses, masses_for_animation

def corr(tall,X,Y,tau,mode):
    M = len(tall)
    dt = tall[1] - tall[0]
    tmax = M - tau
    N = np.size(X[0]) 
    X0 = X[0:tmax,:]
    X2 = 1/tmax*np.sum(X[0:tmax,:]*X[0:tmax,:])
    C = []
    for n in range(tau):
        print(n)
        Xjj = Y[n:n+tmax,:]
        a = np.multiply(np.conjugate(X0),Xjj)
        b = 1/(tmax) * np.sum(a,axis=0)#/X2
        if (mode=='projected'):
            c = b
        else:
            c = np.sum(b)
        C.append(c)
    C = np.array(C)
    t = np.arange(0,tau)*dt
    freq = np.fft.fftfreq(tau,d=dt)
    Z = np.fft.fft(C,axis=0)
    return t, C, freq, Z

def lorentzian(x, x0, A, gamma):
    y = 1/np.pi *  A * 1/2*gamma / ((x - x0)**2 + (1/2*gamma)**2)
    return y

def save(filename, data):
    filename2 = filename
    if os.path.isfile(filename):
        n_of_files = len([name for name in os.listdir('.') if (os.path.isfile(name) and 
                                                               (name==filename or name in [filename+'_'+str(n) for n in range(10)] ) )]) #AHAHAHAHAHAHAHAHAHAHAHAH
        filename2 = filename+'_'+str(n_of_files)
        print(filename, ' already present. Saving it as ', filename2)
    np.savetxt(filename2,data) 
    return

def save_quasiparticles(filename, quasiparticle_kpoints, quasiparticle_info):
    filename2 = filename
    if os.path.isfile(filename):
        n_of_files = len([name for name in os.listdir('.') if (os.path.isfile(name) and 
                                                               (name==filename or name in [filename+'_'+str(n) for n in range(10)] ) )]) #AHAHAHAHAHAHAHAHAHAHAHAH
        filename2 = filename+'_'+str(n_of_files)
        print(filename, ' already present. Saving it as ', filename2)
        
    Nqs = len(quasiparticle_kpoints)  
    file = open(filename2,'ab')
    for i in range(Nqs):
        data1 = quasiparticle_kpoints[i].reshape(1,3)
        data2 = quasiparticle_info[i,:,:]
        np.savetxt(file,data1)
        np.savetxt(file,data2)
    file.close()
    return

def save_append(filename, data1, data2):
    filename2 = filename
#    if os.path.isfile(filename):
#        n_of_files = len([name for name in os.listdir('.') if (os.path.isfile(name) and name==filename)])
#        filename2 = filename+'_'+str(n_of_files)
#        print(filename, ' already present. Saving it as ', filename2)
        
    file = open(filename2,'ab')
    np.savetxt(file,data1)
    np.savetxt(file,data2)
    file.close()
    return

def max_freq(dt_ps, tau):
    #you want the max frequency plotted be 25 Thz
    max_freq = 0.5*1/dt_ps
    if (max_freq < 25):
        meta = int(tau/2)
    else:
        meta = int(tau/2*25/max_freq)
    return meta

def fit_to_lorentzian(x_data, y_data, k, n):
    try:
        if(n in [0,1,2] and np.allclose(k, [0,0,0])): #if acoustic modes at Gamma, don't fit anything
                popt, pcov = np.array([0.0001, 0, 0.0001]), np.zeros((3,3))
        else:
            popt, pcov = curve_fit(lorentzian, x_data, y_data)
    except RuntimeError:
        print('\t\tWasnt able to fit to Lorentzian mode '+str(n)+'\n\n')
        x0 = x_data[np.argwhere(y_data==y_data.max())]
        y0 = y_data.max()
        popt, pcov = np.array([x0,y0,0.0001]), np.zeros((3,3))
    perr = np.sqrt(np.diag(pcov))
    return popt, perr

def odd_num_steps(Num_timesteps):
    if((Num_timesteps-1)%2 == 0): 
        Num_steps = Num_timesteps -1
    return Num_steps

def create_Tkt(Num_timesteps, tot_atoms_uc, N1N2N3, Vt, R0, k):
    N = tot_atoms_uc*N1N2N3
    Vcoll = np.zeros((Num_timesteps,tot_atoms_uc*3),dtype=complex)  
    for j,h,l in zip(range(tot_atoms_uc*3),np.repeat(range(0,N),3)*N1N2N3*3,np.tile(range(0,3),tot_atoms_uc)):
        vels = np.array(Vt[:,h+l:h+N1N2N3*3:3],dtype=complex)
        poss = R0[h:h+N1N2N3*3:3,:]
        x = np.multiply(vels,np.exp(-1j*np.dot(poss,k)))
        Vcoll[:,j] = np.sum(x,axis=1)
    Tkt = Vcoll  
    return Tkt


def pair_distr(file_initial_conf, Rt, Nsteps, dr):
    N = len(Rt[0,0::3])
    ang_to_bohr = 1.8897259886
    SCell = np.genfromtxt(file_initial_conf, skip_header=2, max_rows=3)*ang_to_bohr
    A1, A2, A3 = SCell[0,:], SCell[1,:], SCell[2,:]
    V = np.dot(A1, np.cross(A2, A3))
    L = V**(1/3)
#    dr = .01 #[B] 
    r = np.arange(0,L/2,dr)
    Nbins = len(r)-1
    
    HIST = np.zeros(Nbins)
    for t in range(Nsteps):
        print(t)
        hist = np.zeros(Nbins)
        Rx, Ry, Rz = Rt[t,0::3], Rt[t,1::3], Rt[t,2::3]
#        vtx, vty, vtz = Vt[t,0::3],  Vt[t,1::3], Vt[t,2::3]
    #    R0x, R0y, R0z = R0[::3,0], R0[::3,1], R0[::3,2]
    
        Rmatx, Rmaty, Rmatz = np.tile(Rx,(N,1)).T, np.tile(Ry,(N,1)).T, np.tile(Rz,(N,1)).T
        Rijx, Rijy, Rijz = Rmatx - Rx, Rmaty - Ry, Rmatz - Rz
    
    
        for i in range(N):
    #        vi = np.sqrt(vtx[i]**2+vty[i]**2+vtz[i]**2)
            for j in range(N):
    #            vj = np.sqrt(vtx[j]**2+vty[j]**2+vtz[j]**2) 
                if(i==j):
                    continue
                rijx, rijy, rijz = Rijx[i,j], Rijy[i,j], Rijz[i,j]
                rij = np.sqrt(rijx**2+rijy**2+rijz**2)
                BIN = int(rij/dr) 
                if (BIN < Nbins):
                    hist[BIN] = hist[BIN] + 3 #3*vi*vj 
        HIST = HIST + hist
                        
    const = 4/3 * np.pi* N/V  
    g = np.zeros(Nbins)
    for b in range(Nbins):
        rlower = b*dr
        rupper = (b+1)*dr
        nid = const*(rupper**3-rlower**3)
        g[b] = HIST[b]/N/Nsteps/nid
        
    return r[0:-1], g


def get_real_branches(freqs, eigvecs, masses, ks):
    """
    This code finds the phonon bands based on the eigenvectors. 
    It is equivalent to Phonopy's band connection = .true.
    It is quite hard to read, to be honest I think I don't know myself!
    """
    num_ks = len(ks)
    num_branches = len(freqs[:,0])    
        
    new_indexes = range(num_branches) #you have to start from something, you assume the first kpoint is in order
    all_indexes = [new_indexes]
    
    for k in range(0,num_ks-1):
        kpoint = ks[k]
        next_kpoint = ks[k+1]

        this_eigvecs_T = eigvecs[k].T/np.sqrt(masses)
        norm_this_eigvecs = np.linalg.norm(this_eigvecs_T,axis=1)
        this_eigvecs_norm = this_eigvecs_T.T /norm_this_eigvecs
        
        next_eigvecs_T = eigvecs[k+1].T/np.sqrt(masses)
        norm_next_eigvecs = np.linalg.norm(next_eigvecs_T,axis=1)
        next_eigvecs_norm = next_eigvecs_T.T /norm_next_eigvecs
        
        indexes = []
        
        # for l in new_indexes:
        #     print(l)
        #     eig  = this_eigvecs_norm[:,l]
        #     # print(np.round(eig,2))
        #     dot = np.abs(np.dot(next_eigvecs_norm.T, eig))
        #     print(np.round(dot,2))
        #     index = int(np.argwhere(dot == dot.max()))
        #     if (index in indexes): #case of degeneracy
        #         dot[index] = 0
        #         index = int(np.argwhere(dot==dot.max()))
        #     indexes.append(index)
            
        this_eigvecs_reshaped = this_eigvecs_norm[:, new_indexes]
        
        dot = np.abs(np.dot(this_eigvecs_reshaped.T, next_eigvecs_norm)) #* (np.dot(next_eigvecs_norm.T, this_eigvecs_reshaped)).conj()
        for i in range(num_branches):
            row = dot[i,:]
            max_index, max_row = int(np.argwhere(row == max(row))), max(row)
            for j in range(num_branches):
                if(i==j):
                    continue
                this_row = dot[j,:]
                max_index_this_row, max_this_row = int(np.argwhere(this_row == max(this_row))), max(this_row)
                if(max_index_this_row == max_index):
                    if(max_row > max_this_row):
                        dot[j,max_index_this_row] = 0
                    else:
                        dot[i,max_index] = 0
                            

        maxima = np.max(dot, axis=1)
        for l in range(num_branches):
            index = int(np.argwhere(dot[l,:] == maxima[l]))
            indexes.append(index)
            new_indexes = indexes #update new indexes
            
        all_indexes.append(indexes)    

    freqs_sorted = np.copy(freqs)
    for j in range(1,num_ks):
        freqs_sorted[:,j] = freqs[all_indexes[j],j]
    
    return np.array(all_indexes), freqs_sorted


def get_real_branches_old(freqs, eigvecs, masses, ks):
    num_ks = len(ks)
    num_branches = len(freqs[:,0])    
    
    freqs_until_now = freqs[:,0]
    indexes = np.tile(np.arange(num_branches), (2,1))
    new_indexes = np.arange(num_branches)
    
    for k in range(1):#(1,num_ks-1):
        kpoint = ks[k]
        next_kpoint = ks[k+1]
        
        # if(np.allclose(next_kpoint,[0,0,0])):
        #     next_kpoint = [0,0,0]
        # else:
        #     next_kpoint = next_kpoint/np.linalg.norm(next_kpoint)
#        print(k,qpoints_scaled[k], np.round(next_kpoint,2))

        this_eigvecs = eigvecs[k].T/np.sqrt(masses)*10
        next_eigvecs = eigvecs[k+1].T/np.sqrt(masses)*10
        next_eigvecs = next_eigvecs/np.linalg.norm(next_eigvecs, axis=0)

#        print(np.round(next_eigvecs[0::],2))

        temp = new_indexes#np.arange(num_branches)
        for j,l in zip(range(num_branches), new_indexes):
            eig = this_eigvecs[l,:]
            eig = eig/np.linalg.norm(eig)
#            print(l,np.round(eig,2), freqs[l,k])
            dot_prod = np.abs(np.dot(next_eigvecs,eig))
            print(np.round(dot_prod,2))


            new_index = np.argwhere([dot_prod.max()-0.0000000000001 <= dd <= dot_prod.max() for dd in dot_prod])
            degenerate = len(new_index)
#            print(new_index)
            if (len(new_index) > 1):
#                print('two are degenerate')
#                next_eigvecs_deg = next_eigvecs[new_index,:]
#                dot_nexteigvecs_nextk = np.dot(next_eigvecs_deg.reshape(degenerate*2,3), next_kpoint)
#                dot_nexteigvecs_nextk = np.abs(dot_nexteigvecs_nextk.reshape(degenerate,2))
#                dot_nexteigvecs_nextk = dot_nexteigvecs_nextk/np.linalg.norm(dot_nexteigvecs_nextk)
#                dot_thiseig_nextk = np.abs(np.dot(eig.reshape(1*2,3), next_kpoint))
#                dot_thiseig_nextk = dot_thiseig_nextk/np.linalg.norm(dot_thiseig_nextk)
#                diff = np.abs(dot_thiseig_nextk - dot_nexteigvecs_nextk)
#                diff_sum = np.sum(diff, axis=1)
#                print(np.round(dot_nexteigvecs_nextk,2))
#                print(np.round(dot_thiseig_nextk,2))
#                print('diffsum', np.round(diff_sum,2))
                a=0
                if(a==10):#(np.isclose(dot_nexteigvecs_nextk, 1,  rtol=0.2, atol=0.2).any()):
                    #you have a longitudinal branch
                    print('theres a longitudinal')
#                    isclose = np.isclose(dot_nexteigvecs_nextk, dot_thiseig_nextk, rtol=0.2, atol=.2)
#                    true_long = np.all(isclose, axis=1)
#                    arg_long = np.argwhere(true_long)
#                    new_index = new_index[arg_long]
#                    print(new_index)
#                    
                else:
#                    c = np.argwhere(diff_sum==diff_sum.min())
                    new_index = l#new_index[c]
                    print(new_index)

                
                
            if (new_index in temp[0:j]): #it happens at Gamma where TA modes are degenerate
#                print('aaaa')
                new_index = l

            temp[j] = new_index
#            print()
        new_indexes = temp.astype(int)
#        print(new_indexes)
#        print()



        indexes = np.vstack((indexes,new_indexes))
        
    indexes = indexes.astype(int)
#    print(indexes)
    freqs_sorted = []
    for j in range(num_ks):
        freqs_sorted.append(freqs[indexes[j],j])   
        
    return indexes, np.array(freqs_sorted).T


#fig,ax = plt.subplots()
#    
#masses = np.array([24.305 ,  24.305  , 24.305 ,  15.9994 , 15.9994  ,15.9994])
#xd = np.copy(freqs_disp.T)
#s=get_real_branches(xd, eigvecs_phonopy, masses, Hk,  qpoints_scaled)
#print(np.shape(freqs_disp))
#
#kk = np.arange(len(freqs_disp[:,2]))
#for i in range(6):
#    ax.scatter(kk,freqs_disp[:,i], c='black')
##
##ax.plot(np.arange(len(s[0,:])),s[0,:].flatten())
##ax.plot(np.arange(len(s[0,:])),s[1,:].flatten())
##ax.plot(np.arange(len(s[0,:])),s[2,:].flatten())
#
#ax.plot(np.arange(len(s[0,:])),s[3,:].flatten())
#ax.plot(np.arange(len(s[0,:])),s[4,:].flatten())
#ax.plot(np.arange(len(s[0,:])),s[5,:].flatten())
    
def freqs_spectrum(freq,ZQS):
    num_modes = len(ZQS[0,:,0])
    num_ks = len(ZQS[0,0,:])
#    fig,ax = plt.subplots()
    freqs_from_spectrum = np.zeros((num_modes, num_ks))
    for i in range(num_ks):
        for j in range(num_modes):
            phonon = ZQS[:,j,i]
            if (-.001<phonon.max()<.001):
                continue
#            ax.plot(freq, phonon)
            freq_max = freq[np.argwhere(phonon==phonon.max())]
            freqs_from_spectrum[j,i] = freq_max
#    ax.plot(np.arange(2540), ZQS[:,0,1])
#    ax.plot(np.arange(2540), ZQS[:,0,2])
#    ax.plot(np.arange(2540), ZQS[:,0,3])
    return freqs_from_spectrum.T

def find_Gammas(ks_path):
    ks_path = np.array(ks_path)
    num_ks = len(ks_path)
    kk = np.linspace(0,1,num_ks)
    indexes_Gammas = np.argwhere(np.all(np.isclose(ks_path,[0,0,0]), axis=1)).flatten()
    if(indexes_Gammas[-1]==len(kk)):
        indexes_Gammas_withboundaries = indexes_Gammas
    else:
        indexes_Gammas_withboundaries = indexes_Gammas
        indexes_Gammas_withboundaries = np.concatenate((indexes_Gammas_withboundaries,[len(kk)-1]))
    num_Gammas = len(indexes_Gammas)
    return num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries

def gaussian(x,A, B, c):
#    y = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((x-mi)/sig)**2)
    y = A * np.exp(-B*(x-c)**2) 
    return y

def norm_gaussian(x, mi, sig):
    y = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((np.subtract.outer(x,mi))/sig)**2)
    return y

def get_Force_Constants(eigvecs, freqs_spectrum, ks, R0):
    num_ks = len(freqs_spectrum[0,:])
    num_modes = len(freqs_spectrum[:,0])
    tot_atoms_uc = int(num_modes/3)
    N = len(R0[:,0])
    N1N2N3 = int(N/tot_atoms_uc)
    Dyna_matrixes = np.zeros((num_modes,num_modes, num_ks))
    K = np.zeros((num_modes,N*3))
    summa = 0
    print(np.shape(ks))
    for k in range(1):
        this_k = ks[k,:]
        this_eig = eigvecs[k]
        lamda = np.diag(freqs_spectrum[:,k])
        D = np.dot(np.dot(this_eig,lamda),this_eig.T)
        Dyna_matrixes[:,:,k] = D
    
    for a in range(tot_atoms_uc):
        R_origin = R0[a*N1N2N3,:]
        for b in range(tot_atoms_uc):
            somma = np.zeros((3,3))
            for k in range(num_ks):
                j = k + N1N2N3*a
                R_next = R0[j,:]
                diffR = R_next - R_origin
                kdotr = np.dot(this_k, diffR)
                D = Dyna_matrixes[:,:,k]
                Da = D[a*3:a*3+3, b*3:b*3+3]
                
                somma = somma + Da*np.exp(1j*kdotr)
  
        
    return

def all_kpoints(N1,N2,N3, Hk):
    N1N2N3 = N1*N2*N3
    #Compute all possible k points
    x1 = np.arange(0,N1)/N1
    x2 = np.arange(0,N2)/N2
    x3 = np.arange(0,N3)/N3
    comb = np.array(np.meshgrid(x1,x2,x3)).T.reshape(-1,3)
    allkpoints_scaled = comb.T
    allkpoints = np.dot(Hk,allkpoints_scaled)
    return allkpoints_scaled.T, allkpoints.T

def get_K_renorm_old(tot_atoms_uc, N1, N2, N3, H, R0, kpoints, Ds):
    """
    old getK_renorm, without exploiting irreducible ks
    """
    N1N2N3 = N1*N2*N3
    N1N2N3_k = len(kpoints)
    N = N1N2N3 * tot_atoms_uc
    R_mat = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
    
    K_renorm = np.zeros((tot_atoms_uc*3, N*3), dtype=complex)
    for i in range(0,tot_atoms_uc*3,3):
        atom_ref_i = int(i/3)
        count = 1
        for j in range(N1N2N3*tot_atoms_uc):
            atom_ref_j = int(j/N1N2N3)
            R = R_mat[i,j*3:j*3+3]
            KR = sum_Dqs_old(R,kpoints,Ds, N1N2N3_k)
            K_renorm[i:i+3,j*3:j*3+3] = KR[i:i+3,atom_ref_j*3:atom_ref_j*3+3]
    return  K_renorm.real/(0.964*10**(4))

def get_K_renorm(tot_atoms_uc, N1, N2, N3, H, R0, irr_ks, permutations_types, Ds):
    N1N2N3_k = len(Ds[:,0,0])
    N1N2N3 = N1*N2*N3
    N = N1N2N3 * tot_atoms_uc
    R_mat = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
    
    K_renorm = np.zeros((tot_atoms_uc*3, N*3), dtype=complex)
    for i in range(0,tot_atoms_uc*3,3):
        atom_ref_i = int(i/3)
        count = 1
        for j in range(N1N2N3*tot_atoms_uc):
            atom_ref_j = int(j/N1N2N3)
            R = R_mat[i,j*3:j*3+3]
            KR = sum_Dqs(R,irr_ks, permutations_types, Ds, N1N2N3)
            K_renorm[i:i+3,j*3:j*3+3] = KR[i:i+3,atom_ref_j*3:atom_ref_j*3+3]
    K_renorm = K_renorm/(0.964*10**(4)) #/0.529177249
    print('Maximum imaginary part of renorm FORCE CONSTANTS:', np.max(K_renorm.imag), '[ev/B]')
    return  K_renorm.real




def sum_Dqs_old(R,kpoints,Ds, N1N2N3):
    """
    old sum Dqs, without exploiting irr ks
    """
    kpoints2 = -kpoints
    # kpoints = np.vstack((kpoints,kpoints2))
    Ds2 = np.conjugate(Ds[:,:,:])
    # Ds = np.concatenate((Ds, Ds2), axis=0)
    kdotR = np.dot(kpoints,R)
    
    cs = np.exp(1j*kdotR).reshape(N1N2N3,1,1)

    A = 1/N1N2N3*np.multiply(cs,Ds)
    Kr = np.sum(A, axis=0)
    return Kr

def sum_Dqs(R,irr_kpoints, permutations_types, Ds, N1N2N3):
    N1N2N3_irr = len(irr_kpoints)
    permutations = np.array([[[0,1,0],[1,0,0],[0,0,1]], 
                             [[0,0,1],[0,1,0],[1,0,0]], 
                             [[1,0,0],[0,0,1],[0,1,0]],
                             [[0,1,0],[0,0,1],[1,0,0]],
                             [[0,0,1],[1,0,0],[0,1,0]]])
    shape_D = np.shape(Ds[0,:,:])
    tot_atoms_uc = int(shape_D[0]/3)
    
# =============================================================================
# irreducible part
    #              this is in case you wanna force it real
    # irr_kpoints2 = -irr_kpoints
    # irr_kpoints_doubled = np.vstack((irr_kpoints,irr_kpoints2))
    # Ds2 = np.conjugate(Ds[:,:,:])
    # Ds = np.concatenate((Ds, Ds2), axis=0)
    
    kdotR = np.dot(irr_kpoints,R)
    cs = np.exp(1j*kdotR).reshape(N1N2N3_irr,1,1)
    A = 1/N1N2N3*np.multiply(cs,Ds)
    Kr_irr = np.sum(A, axis=0)
# =============================================================================

# =============================================================================
# other non irreducible kpoints
    Ds_non_irr, ks_non_irr = [], []
    for i in range(len(irr_kpoints)):
        this_irr_k = irr_kpoints[i]
        D_irr_k = Ds[i]
        this_permutations = permutations_types[i]
        for permut in this_permutations:
            matrix_perm = permutations[permut]
            non_irr_k = np.dot(matrix_perm, this_irr_k)
            ks_non_irr.append(non_irr_k)
            non_irr_D = np.zeros(shape_D, dtype=complex)
            for l in range(tot_atoms_uc):
                for m in range(tot_atoms_uc):
                    D_3by3 = D_irr_k[l*3:l*3+3, m*3:m*3+3]
                    non_irr_D[l*3:l*3+3, m*3:m*3+3] = np.dot(np.dot(matrix_perm, D_3by3), matrix_perm.T)
            Ds_non_irr.append(non_irr_D)   
    Ds_non_irr = np.array(Ds_non_irr)
    ks_non_irr = np.array(ks_non_irr)
    N1N2N3_non_irr = len(ks_non_irr)
    
    #              this is in case you wanna force it real
    # non_irr_kpoints2 = -ks_non_irr
    # non_irr_kpoints = np.vstack((ks_non_irr,non_irr_kpoints2))
    # Ds_non_irr2 = np.conjugate(Ds_non_irr[:,:,:])
    # Ds_non_irr = np.concatenate((Ds_non_irr, Ds_non_irr2), axis=0)
    
    kdotR_non_irr = np.dot(ks_non_irr,R)
    cs_non_irr = np.exp(1j*kdotR_non_irr).reshape(N1N2N3_non_irr,1,1)
    A_non_irr = 1/N1N2N3*np.multiply(cs_non_irr,Ds_non_irr)
    Kr_non_irr = np.sum(A_non_irr, axis=0)
# =============================================================================

    Kr = Kr_irr + Kr_non_irr

    
    return Kr




def get_Ds(N1,N2,N3,Hk, K, R0, masses_uc, tot_atoms_uc, H):
    """
    old, doesn't renormalise
    """
    x,y,z = np.linspace(0,1,N1+1)[0:-1], np.linspace(0,1,N2+1)[0:-1], np.linspace(0,1,N3+1)[0:-1]
    k_scal = np.array(np.meshgrid(x,y,z)).T.reshape(-1,3)
    kpoints = np.dot(Hk,k_scal.T).T
    Ds = []
    for k in kpoints:
        D = get_D(k, K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, H)
        Ds.append(D)
    Ds = np.array(Ds)
    return kpoints, Ds








def get_R_FC(N1o, N2o, N3o, N1,N2,N3,tot_atoms_uc, Suc, SCell):
    N1N2N3 = N1*N2*N3
    N = N1N2N3*tot_atoms_uc
    
    N1N2N3o = N1o*N2o*N3o
    if(N1N2N3 != N1N2N3o):
        conv_factor = np.array([N1o/N1, N2o/N2, N3o/N3])
    else:
        conv_factor=1
    h = SCell/conv_factor
    

    COMB = np.zeros((tot_atoms_uc*N1N2N3,3))
    for atom in range(tot_atoms_uc):
        x0,y0,z0 = Suc[atom,0],Suc[atom,1],Suc[atom,2]
        x,y,z = np.linspace(0,1,N1+1)[0:-1], np.linspace(0,1,N2+1)[0:-1], np.linspace(0,1,N3+1)[0:-1]
        comb = np.array(np.meshgrid(x+x0/N1,y+y0/N2,z+z0/N3)).T.reshape(-1,3)
        COMB[atom*N1N2N3:(atom+1)*N1N2N3,:] = comb
    
    R_FC_normal = np.dot(h, COMB.T).T
    R_FC = np.zeros((tot_atoms_uc*3,N*3))  
    count = 0
    
    for atom in range(tot_atoms_uc):
        r_fc = np.dot(h,COMB.T).T
        x0,y0,z0 = Suc[atom,0],Suc[atom,1],Suc[atom,2]
        atom_origin = np.dot(h,[x0/N1,y0/N2,z0/N3])
        
        row = (r_fc-atom_origin).flatten()
        repeated_row = np.tile(row,(3,1))
        R_FC[count:count+3,:] = repeated_row
        count =  count + 3

    return h, R_FC_normal, R_FC

def get_Scells(H, R0):
    N = len(R0)
    x1 = np.arange(-1,2)
    comb = np.array(np.meshgrid(x1,x1,x1)).T.reshape(-1,3)
    displ = np.dot(H, comb.T).T
    Scells = np.zeros((27, N, 3))
    for c in range(27):
        Scells[c] = R0 + displ[c]     
    return Scells

def R_matrix(N1,N2,N3,tot_atoms_uc,R0, H):
    """
    returns the R matrix
    """
    N1N2N3 = N1 * N2 * N3
    N = len(R0[:,0])
    Scells = get_Scells(H, R0)
    
    
    R_matrix = np.zeros((tot_atoms_uc*3,N*3))
    count = 0
    for i in range(tot_atoms_uc):
        atom_origin = R0[i*N1N2N3]
        row = []
        for j in range(N):
            dist_Scells_vec = Scells[:,j,:] - atom_origin
            dist_Scells = np.linalg.norm(dist_Scells_vec, axis=1)
            found_min_dist = np.argwhere(dist_Scells == np.min(dist_Scells, axis=0))
            found_min_dist_0 = found_min_dist[0]
            row.append((Scells[found_min_dist_0,j,:] - atom_origin).flatten())
        row = np.array(row)
        
        repeated_row = np.tile(row.flatten(),(3,1))
        R_matrix[count:count+3,:] = repeated_row
        count =  count + 3
        
    return R_matrix#*0.529177249

def get_D(k, K,N1,N2,N3,R0,masses,tot_atoms_uc, H):
    """
    builds the dynamical matrix
    expects forces in 1/amu * ev / A**2
    returns Thz**2
    """
    N1N2N3 = N1 * N2 * N3
    N = N1N2N3*tot_atoms_uc
    
    R = R_matrix(N1,N2,N3,tot_atoms_uc, R0, H)
    #prepare kdotr
    kdotr = np.zeros((tot_atoms_uc*3,N*3))
    for i in range(tot_atoms_uc*3):
        for j in range(0,N*3,3):
            kdotr[i,j] = np.dot(k,R[i,j:j+3])
            kdotr[i,j+1] = np.dot(k,R[i,j:j+3])
            kdotr[i,j+2] = np.dot(k,R[i,j:j+3])
    
    D = np.zeros((tot_atoms_uc*3,tot_atoms_uc*3),dtype=complex)
    for i in range(tot_atoms_uc*3):
        for j,h in zip(range(tot_atoms_uc*3),np.repeat(range(0,N),3)):
#            print(i,j,h)
            mass_coeff = complex(1/(masses[i]*masses[j])**(0.5))
            exp = 0*1j
#            h = 1
            for l in range(j+h*3*(N1N2N3-1),j+h*3*(N1N2N3-1)+(N1N2N3)*3,3):
#                print(i,l,np.round(np.exp(1j*kdotr[i,l]),2),K[i,l])
                exp = exp + K[i,l]*np.exp(1j*kdotr[i,l])
#            print()
            D[i,j] = mass_coeff*exp
    return (D+D.conj().transpose())/2*0.964*10**(4)#, np.exp(1j*kdotr)#, R



def voigt_eq(n, k, freq, Z, sig):
    if(n in [0,1,2] and np.allclose(k, [0,0,0])): #if acoustic modes at Gamma, don't fit anything
        popt, pcov = np.array([0, 0, 0]), np.zeros((3,3))
        return popt[0], popt[-1]
    
    mi = freq[int(len(freq)/2)]
    gaussian = norm_gaussian(freq, mi, sig)                
    convolution = signal.fftconvolve(Z,gaussian, mode='same') / sum( gaussian)
    
    
    f_max = int(np.argwhere(convolution==convolution.max()))
    conv_sx, conv_dx = convolution[0:f_max], convolution[f_max::]
    if(len(np.argwhere(conv_sx < convolution.max()/2)) > 0):
        sx, dx = np.argwhere(conv_sx < convolution.max()/2)[-1], np.argwhere(conv_dx < convolution.max()/2)[0] + f_max
        fv = (freq[dx] - freq[sx])[0]
    else:
        dx = np.argwhere(conv_dx < convolution.max()/2)[0] + f_max
        fv = freq[dx][0]*2
    fv = (freq[dx] - freq[sx])[0]
    fg = 2*sig*np.sqrt(2*np.log(2))
    a, b, c = 0.0692, -1.0692*fv, -fg**2+fv**2
    fl = (-b/2-np.sqrt((b/2)**2-a*c))/a 
#   www = (-fv**2+fg**2)/(-fv)
    return freq[f_max], fl






def all_freqs(N1,N2,N3, Hk, Num_timesteps, tot_atoms_uc, Vt):
    N1N2N3 = N1*N2*N3
    meta = int(len(freq)/2)
    freq_meta = freq[0:meta]
    #Compute all possible k points
    x1 = np.arange(0,N1+1)/N1
    x2 = np.arange(0,N2+1)/N2
    x3 = np.arange(0,N3+1)/N3
    comb = np.array(np.meshgrid(x1,x2,x3)).T.reshape(-1,3)
    #a1 = np.multiply(comb,b1)
    #a2 = np.multiply(comb,b2)
    #a3 = np.multiply(comb,b3)
    #kpoints = a1+a2+a3
    allkpoints_scaled = comb.T
    allkpoints = np.dot(Hk,allkpoints_scaled)
    
    import matplotlib.pyplot as plt
    from scipy import signal
    for k in range(1,2):
        fig,ax = plt.subplots()
        #Creating the collective variable based on the k point
        kpoint = allkpoints[:,k]
        Tkt = create_Tkt(Num_timesteps-1, tot_atoms_uc, N1N2N3, Vt, R0, kpoint)
        Tkw = np.fft.fft(Tkt, axis=0)
        Sq = (np.sum(np.conjugate(Tkw)*Tkw, axis=1)).real*cev/(kbev*T)/Num_timesteps*dt_ps
        sig, mi = .1/2, freq[int(meta/2)]
        gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq_meta-mi)/sig)**2)
        convolution = signal.fftconvolve(Sq[0:meta],gaussian, mode='same') / sum( gaussian)
        print(convolution)
        ax.plot(freq_meta,Sq[0:meta], freq_meta, convolution)
        area_q = np.trapz(Sq, freq[0:meta])
        print('\t DOS for this kpoint: ', area_q)
    return allkpoints_scaled

#SSS= all_freqs(10,10,10, Hk, Num_timesteps, tot_atoms_uc, Vt)
#R = R0[0::3,:]
#get_Force_Constants(eigvecs_phonopy, freqs_from_spectrum.T, ks, R)

def get_nac_params(BORN_file, SCell, N1, N2, N3, tot_atoms_uc):
    eps_inf, BORN = np.genfromtxt(BORN_file, skip_header=1, max_rows=1).reshape(3,3), np.genfromtxt(BORN_file, skip_header=2, max_rows=tot_atoms_uc).reshape(tot_atoms_uc*3,3)
    a1, a2, a3 = SCell[0]/N1, SCell[1]/N2, SCell[2]/N3
    Vuc = np.dot(a1, np.cross(a2,a3))
    return eps_inf, BORN, Vuc

def get_nac_C(kpoint,Vuc, BORN, eps_inf, masses, N1N2N3):
    const = 11.78*1e+06/(2*np.pi) #3.4191*1e+02 #1.18*1e+07
    constevA2 = 1.222*1e+03 / (2*np.pi)
    num = np.outer(np.dot(kpoint,BORN.T) , np.dot(BORN,kpoint))
    den = np.dot(kpoint.T,np.dot(eps_inf,kpoint))
    Mij = np.sqrt(np.outer(masses,masses))
    Cna = 4*np.pi/Vuc * num/den *constevA2 /N1N2N3 
    return Cna

def get_K_corr(K, C_nac):
    K2 = np.copy(K)
    dofs = len(C_nac)
    tot_atoms_uc = int(dofs/3)
    N1N2N3 = int(len(K[0,:])/tot_atoms_uc/3)
    for i in range(tot_atoms_uc):
        this_Cnac = C_nac[:,i*3:i*3+3]
        A = np.tile(this_Cnac,N1N2N3)
        K2[:,i*N1N2N3*3:(i+1)*N1N2N3*3] += A
    return K2







def interpol_spectrum(ks_path, freq, freqs_from_spectrum, kk, kk_many, frequencies, eigvecs, modes, ZQS):
    num_ks = len(ks_path)
    num_modes = len(freqs_from_spectrum[0,:])
    num_freqs = len(ZQS[:,0,0])
    df = freq[1] - freq[0]

    
    ZQS_interp = np.zeros((num_freqs,num_modes, len(kk_many)))
    
    
    ktot = 0
    for k in range(num_ks-1):
        count_k_in_between = 0
        for j in range(num_modes):  
            this_gaussian = ZQS[:,j,k]
            next_gaussian = ZQS[:,j,k+1]
            
            # import matplotlib.pyplot as plt
            # fig,ax = plt.subplots()
#            #to comment
#            popt_this, pcov_this = curve_fit(gaussian, freq, this_gaussian)
#            popt_next, pcov_next = curve_fit(gaussian, freq, next_gaussian)
            
            this_max = freqs_from_spectrum[k,j]
            next_max = freqs_from_spectrum[k+1,j]
            diff_maxima = next_max - this_max
            
            if(diff_maxima >= 0):
                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
                this_gaussian_fit = this_gaussian_interp(freq-diff_maxima)
                next_gaussian_fit = next_gaussian_interp(freq)
                shift_interp = -next_max
            else:
                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
                this_gaussian_fit = this_gaussian_interp(freq)
                next_gaussian_fit = next_gaussian_interp(freq+diff_maxima)
                shift_interp = -this_max
            
#            area_this_gaussian = np.trapz(this_gaussian_fit, freq)
#            area_next_gaussian = np.trapz(next_gaussian_fit, freq)

            this_k = kk[k]
            next_k = kk[k+1]
            indexes_sx, indexes_dx = np.argwhere(this_k<=kk_many).flatten(), np.argwhere(kk_many<next_k).flatten()
            indexes = np.intersect1d(indexes_sx, indexes_dx)
            interm_k = kk_many[indexes]

            # interp = INTERP[sector+j*num_Gammas]
            intermediate_maxima = frequencies[j,indexes]
            count_k_in_between = 0
            for m in range(len(interm_k)):
                mi = intermediate_maxima[m]
                x = (interm_k[m] - interm_k[0])/(next_k-interm_k[0])
#                area_factor = area_this_gaussian+(area_next_gaussian-area_this_gaussian)*x
                interp_gaussian = this_gaussian_fit+(next_gaussian_fit-this_gaussian_fit)*x
                interp_gaussian_interp = interp1d(freq, interp_gaussian,bounds_error=False, fill_value=0)
                
                interp_gaussian_fit = interp_gaussian_interp(freq-(shift_interp+mi))
                ZQS_interp[:,j,ktot+count_k_in_between] = interp_gaussian_fit
                count_k_in_between = count_k_in_between + 1
                

#                popt = popt_this + (popt_next-popt_this)*x
#                interp_gaussian_fit = gaussian(freq, *popt)#1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2) *area_factor

#                 ax.plot(freq, interp_gaussian_fit, label='interp')
#             ax.plot(freq, this_gaussian, label='real this')
#             ax.plot(freq,next_gaussian, label='real next')
# #            ax.plot(freq, this_gaussian_fit, label='real this fit')
# #            ax.plot(freq,next_gaussian_fit, label='real next fit')
#             ax.legend()
            
        ktot = ktot + count_k_in_between
    ZQS_interp[:,:,-1] = ZQS[:,:,-1]
    return ZQS_interp










#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 13 14:12:07 2019

@author: gc4217
"""
import numpy as np
import os, sys

def avg_vs_t(time,quantity):
    avg = []
    for i in range(1,len(time)):
        dt = time[i]-time[i-1]
        y = quantity[0:i]
        integral = np.sum(dt*y,axis=0)
        avg.append(1/(i*dt)*integral)
    avg = np.array(avg)
    return avg

N1, N2, N3 = int(sys.argv[1]), int(sys.argv[2]), int(sys.argv[3])
n_uc = int(sys.argv[4])
N1N2N3 = N1*N2*N3
N = N1*N2*N3*n_uc
flag = sys.argv[5]

print('Hello, Im now converting the ASAP output files...\n')

if(flag=='relax'):
    with open('Cartesian_M0','r') as f:
        A = f.readlines()[0:N+1]
    del A[0]
    
    with open('pos','w') as g:
        g.writelines(A)
        
    R0 = np.loadtxt('pos',usecols=(1,2,3))
    np.savetxt('R0',R0)
    
    os.remove('pos')
    
    print('Finished. R0 created.\n\n')
    print('Please copy the file R0 to your phonon DOS folder!')
    
if(flag=='avg'):
    with open('Cartesian_0','r') as f:
        A = f.readlines()
        
    a = 0
    for i in range(0,len(A),N+1):
        del A[i-a]
        a = a + 1
    with open('pos','w') as g:
        g.writelines(A)
        
    Rall = np.loadtxt('pos',usecols=(1,2,3))
    
    Rsum = np.zeros((N,3))
    n = 0
    for i in range(0,len(Rall),N):
        n = n + 1
        #pos_first_Ba = Rall[i:i+1,:]
        Rsum = Rsum + Rall[i:i+N] #- pos_first_Ba
    Ravg = Rsum/n 
    np.savetxt('R0_avg',Ravg)
    
    os.remove('pos')
    
    print('Finished. R0_avg created.\n\n')
    print('Please copy the file R0_avg to your phonon DOS folder!')































#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jan 24 14:40:14 2020

@author: gc4217
"""
import numpy as np
from scipy import signal
import os, sys
from decomp import plot, read, util

# =============================================================================
# Parameters
input_file =  sys.argv[1]
plot_type = read.read_post_proc(input_file)[0]
Masses = read.read_post_proc(input_file)[1]
n_atom_unit_cell = read.read_post_proc(input_file)[2]
N1, N2, N3 = read.read_post_proc(input_file)[3:6]
file_forces = read.read_post_proc(input_file)[6]
file_SPOSCAR = read.read_post_proc(input_file)[7]
max_Z = read.read_post_proc(input_file)[8]
gaussian_smoothing = read.read_post_proc(input_file)[9]
interp = read.read_post_proc(input_file)[10]
kinput = read.read_post_proc(input_file)[11::][0]
labels = read.read_post_proc(input_file)[12]
modes = read.read_post_proc(input_file)[13]
temperatures_folders = read.read_post_proc(input_file)[14]
NAC = read.read_post_proc(input_file)[15]

K = read.read_FC(file_forces)
BORN_file = 'BORN'


tot_atoms_uc = int(np.sum(n_atom_unit_cell)) 
N1N2N3, N = N1*N2*N3, N1*N2*N3*tot_atoms_uc
Nqs_path, ks_path, ks_scaled_path, distances_comm, x_labels, Hk = util.get_commensurate_kpath(file_SPOSCAR, N1,N2,N3, N1,N2,N3, kinput, labels)
SCell, Ruc, Suc, R0, R0_repeat, S0, masses, masses_uc = read.read_SPOSCAR_and_masses(file_SPOSCAR, n_atom_unit_cell, Masses, N1, N2, N3)  
# =============================================================================

if(plot_type==0):
    print('Generating plot 0...')
    namedir = plot.create_folder('')
    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    for i in range(Nqs_path):
        to_skip = (1+meta)*i
        ZZ = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
        Z_q = ZZ[:,0]
        Z = ZZ[:,1:]
        to_skip2 = 4*i
        Params = np.genfromtxt('quasiparticles',skip_header=to_skip2+1, max_rows=3)
        
        k_scaled = ks_scaled_path[i]
        k = ks_path[i]
        
        # harmonic things
        D = util.get_D(k, K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals = np.linalg.eigvalsh(D)
        idx = np.argsort(eigvals)
        eigvals = eigvals[idx]
        omegas = np.sqrt(eigvals)/2/np.pi

        freq_disp = omegas
        print(' Creating plots for k point = ', k_scaled)
        for n in range(tot_atoms_uc*3):
            params = Params[:,n]
            if(gaussian_smoothing != 0):
                if(gaussian_smoothing < 0):
                    sig, mi = params[-1]/2, freq[int(len(freq)/2)]
                else:
                    sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
            if(sig==0):
                sig = 0.0001

            gaussian = util.norm_gaussian(freq, mi, sig)                
            # convolution_lorentzian = signal.fftconvolve(Z[:,n], plot.lorentzian(freq, freq[int(len(freq)/2)], params[1], params[2]), mode='same') / sum( plot.lorentzian(freq, *params))
            convolution = signal.fftconvolve(Z[:,n],gaussian, mode='same') / sum( gaussian)
            
            plot.save_proj(freq, Z[:,n],Z_q, convolution, ks_scaled_path[i],freq_disp[n],n,namedir, params, max_Z)


if(plot_type==1):
    print('Generating plot 1...')
    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    ZQS = [np.zeros(meta)]
    count = 1
    accepted_qpoints = []
    accepted_labels = []
    for i in range(Nqs_path):
        to_skip = (1+meta)*i
        qpoint = np.genfromtxt('Zqs',skip_header=to_skip+0, max_rows=1)
        for j in range(len(kinput)):
            if(np.allclose(qpoint, kinput[j]) and (qpoint.tolist() not in accepted_qpoints)):
                accepted_qpoints.append(qpoint.tolist())
                accepted_labels.append(labels[j])
                Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)[:,0]
                
                sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
                if(sig<=0):
                    sig = 0.0001

                gaussian = util.norm_gaussian(freq, mi, sig)                
                # convolution_lorentzian = signal.fftconvolve(Z[:,n], plot.lorentzian(freq, freq[int(len(freq)/2)], params[1], params[2]), mode='same') / sum( plot.lorentzian(freq, *params))
                convolution = signal.fftconvolve(Zq,gaussian, mode='same') / sum( gaussian)
                ZQS.append(convolution)
                count = count + 1
            
    ZQS = np.array(ZQS).T
    ZQS[:,0] = np.sum(ZQS[:,1::],axis=1)
    plot.plot1(freq,ZQS, accepted_labels)
    
    
if(plot_type==2): 
    print('Generating plot 2...')
    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    ZQS = np.zeros((meta, tot_atoms_uc*3+1, len(kinput)))
    count = 0
    accepted_qpoints = []
    accepted_labels = []
    freqs_from_disp = np.zeros((tot_atoms_uc*3,len(kinput)))
    for i in range(Nqs_path):
        to_skip = (1+meta)*i
        qpoint = np.genfromtxt('Zqs',skip_header=to_skip+0, max_rows=1)
        to_skip2 = 4*i
        Params = np.genfromtxt('quasiparticles',skip_header=to_skip2+1, max_rows=3)
        
        for j in range(len(kinput)):
            k_scaled = kinput[j]
            k = np.dot(Hk, k_scaled)
            if(np.allclose(qpoint, k_scaled) and (qpoint.tolist() not in accepted_qpoints)):
                accepted_qpoints.append(qpoint.tolist())
                accepted_labels.append(labels[j])
                
                # harmonic things
                D = util.get_D(k, K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
                eigvals = np.linalg.eigvalsh(D)
                idx = np.argsort(eigvals)
                eigvals = eigvals[idx]
                omegas = np.sqrt(eigvals)/2/np.pi

                for m in range(Nqs_path):
                    if(np.allclose(qpoint,ks_scaled_path[m])):
                        freqs_from_disp[:,count] = omegas
                        break
                
                Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
                for n in range(tot_atoms_uc*3):
                    params = Params[:,n]
                    if(gaussian_smoothing != 0):
                        if(gaussian_smoothing < 0):
                            sig, mi = params[-1]/2, freq[int(len(freq)/2)]
                        else:
                            sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
                    if(sig==0):
                        sig = 0.0001
    
                    gaussian = util.norm_gaussian(freq, mi, sig)                
                    # convolution_lorentzian = signal.fftconvolve(Z[:,n], plot.lorentzian(freq, freq[int(len(freq)/2)], params[1], params[2]), mode='same') / sum( plot.lorentzian(freq, *params))
                    convolution = signal.fftconvolve(Zq[:,n+1],gaussian, mode='same') / sum( gaussian)
                    Zq[:,n+1] = convolution
                Zq[:,0] = np.sum(Zq[:,1::], axis=1)
                ZQS[:,:,count] = Zq[:,:]
                count = count + 1
                
    
    plot.plot2(freq,ZQS, modes, accepted_labels, freqs_from_disp)
    
    
if(plot_type==3):
    print('Generating plot 3...')
    subdirectories = [x[1] for x in os.walk(temperatures_folders)][0]
    subdirectories.sort(key= lambda x: float(x.strip('K')))
    Ts = [int(x.strip('K')) for x in subdirectories]
    frequencies = np.zeros((2,tot_atoms_uc*3,len(kinput),len(Ts)))
    count_T = 0
    for subdir in subdirectories:
        count_q = 0
        accepted_qpoints = []
        accepted_labels = []

        for i in range(Nqs_path):
            to_skip = (4)*i
            qpoint = np.genfromtxt(temperatures_folders+subdir+'/quasiparticles',skip_header=to_skip+0, max_rows=1)
            for j in range(len(kinput)):
                if(np.allclose(qpoint, kinput[j]) and (qpoint.tolist() not in accepted_qpoints)):
                    accepted_qpoints.append(qpoint.tolist())
                    accepted_labels.append(labels[j])
                    omegas_gammas = np.genfromtxt(temperatures_folders+subdir+'/quasiparticles',skip_header=to_skip+1, max_rows=3)
                    frequencies[:,:,count_q, count_T] = omegas_gammas[0::2,:]
                    count_q = count_q + 1
        count_T = count_T + 1
    
    diff =len(frequencies[0,0,:,0]) -  len(accepted_labels) 
    if(diff != 0): #this is to eliminate repetitive kpoints if present
        for h in range(diff):
            frequencies = np.delete(frequencies, -1, axis=2)
    plot.plot3(Ts, frequencies, modes, accepted_labels)

   
if(plot_type==4):
    print('Generating plot 4...')
    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    K_renorm = np.loadtxt('K_renorm')
    
    # take the path the user inputs and find all the k points you can plot
    ks_scal_path_many, ks_path_many, distances = util.get_kpath_many(ks_scaled_path, Hk, npoints=interp+1)
    
    # SCell_fc, R0_fc, R_fc = util.get_R_FC(N1, N2, N3, 4,4,4,tot_atoms_uc, Suc, SCell) 
# =============================================================================
    freq_DOS, DOS, DOS_renorm = np.arange(0,25,0.01), np.zeros(len(np.arange(0,25,0.01))), np.zeros(len(np.arange(0,25,0.01)))
    kpoints_scal, kpoints = util.get_kgrid(interp,interp,interp, Hk)
    N1N2N3_k = interp*interp*interp
    irr_ks, irr_ks_sc, weights, permutation_types = util.get_irr_ks(kpoints, kpoints_scal)    
    for i in range(len(irr_ks)):
        k = irr_ks[i]
        weight = weights[i]
        print('\t processing DOS k-point:', i, irr_ks_sc[i])
        # Renormalised          
        D = util.get_D(k, K_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_renorm, omegas_renorm = util.get_eigvals(D)
        # 0K dispersion
        D_0K = util.get_D(k, K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_0K, omegas_0K = util.get_eigvals(D_0K)

        sigma = gaussian_smoothing
        DOS = DOS + np.sum(util.norm_gaussian(freq_DOS,omegas_0K,sigma), axis=1)*weight/(3*tot_atoms_uc*N1N2N3_k*2)
        DOS_renorm = DOS_renorm + np.sum(util.norm_gaussian(freq_DOS,omegas_renorm,sigma), axis=1)*weight/(3*tot_atoms_uc*N1N2N3_k*2)
    DOS_info = np.vstack((freq_DOS, DOS, DOS_renorm)).T
# =============================================================================
    
# =============================================================================    
    count_k = 0
    frequencies = np.zeros((1,tot_atoms_uc*3,len(ks_path_many)))
    frequencies_disp = np.zeros((1,tot_atoms_uc*3,len(ks_path_many)))
    EIGVECS, EIGVECS_0K = [], []
    for i in range(len(ks_path_many)):
        k = ks_path_many[i]
        
        #NAC
        if NAC:
            if(np.allclose(k, [0,0,0])):
                kpoint_nac = ks_path_many[i+1]
            else:
                kpoint_nac = k
            eps_inf, BORN, Vuc = util.get_nac_params(BORN_file, SCell, N1, N2, N3, tot_atoms_uc)
            C_nac = util.get_nac_C(kpoint_nac, Vuc, BORN, eps_inf, masses_uc, N1N2N3)
            K_corr_0K, K_corr_renorm = util.get_K_corr(K, C_nac), util.get_K_corr(K_renorm, C_nac)
        else:
            K_corr_0K, K_corr_renorm = K, K_renorm
        
        print('\t processing k-point:', i, ks_scal_path_many[i])
        # Renormalised          
        D_renorm = util.get_D(k, K_corr_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_renorm, eigvecs_renorm, omegas_renorm = util.get_eigvals_eigvec(D_renorm)
        # 0K dispersion
        D_0K = util.get_D(k, K_corr_0K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_0K, eigvecs_0K, omegas_0K = util.get_eigvals_eigvec(D_0K)
        
        EIGVECS_0K.append(eigvecs_0K)
        EIGVECS.append(eigvecs_renorm)
            
        frequencies[:,:,count_k] = omegas_renorm 
        frequencies_disp[:,:,count_k] = omegas_0K 
        count_k = count_k + 1
# =============================================================================
        
        
    # this is to reshape spectra according to real branches
    indexes, freqs_reshaped = util.get_real_branches(frequencies[0,:,:], EIGVECS, masses_uc, ks_path_many)
    indexes, freqs_disp_reshaped = util.get_real_branches(frequencies_disp[0,:,:], EIGVECS_0K, masses_uc, ks_path_many)
    frequencies_reshaped, frequencies_disp_reshaped  = np.copy(frequencies), np.copy(frequencies_disp)
    frequencies_reshaped[0,:,:], frequencies_disp_reshaped[0,:,:] = freqs_reshaped, freqs_disp_reshaped

    plot.plot4(distances, distances_comm, frequencies_reshaped, frequencies_disp_reshaped,  x_labels, DOS_info)
    
    
if(plot_type==40):
    print('Generating plot 40...')
    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    
    # take the path the user inputs and find all the k points you can plot
    ks_scal_path_many, ks_path_many, distances = util.get_kpath_many(ks_scaled_path, Hk, npoints=interp+1)
    
    K_renorm = np.loadtxt('K_renorm')
    # SCell_fc, R0_fc, R_fc = util.get_R_FC(N1, N2, N3, 4,4,4,tot_atoms_uc, Suc, SCell) 
            
# =============================================================================
    freq_DOS, DOS = np.arange(0,25,0.01), np.zeros(len(np.arange(0,25,0.01)))
    kpoints_scal, kpoints = util.get_kgrid(interp, interp, interp, Hk)
    N1N2N3_k = interp*interp*interp
    irr_ks, irr_ks_sc, weights, permutation_types = util.get_irr_ks(kpoints, kpoints_scal)
    for i in range(len(irr_ks)):
        k = irr_ks[i]
        weight = weights[i]
        print('\t processing DOS k-point:', i, irr_ks_sc[i])
        # 0K dispersion
        D_0K = util.get_D(k, K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_0K, omegas_0K = util.get_eigvals(D_0K)
        sigma = gaussian_smoothing
        DOS = DOS + np.sum(util.norm_gaussian(freq_DOS,omegas_0K,sigma), axis=1)*weight/(3*tot_atoms_uc*N1N2N3_k*2)
    DOS_info = np.vstack((freq_DOS, DOS)).T
# =============================================================================
   
# =============================================================================            
    count_k = 0
    frequencies = np.zeros((1,tot_atoms_uc*3,len(ks_path_many)))
    frequencies_disp = np.zeros((1,tot_atoms_uc*3,len(ks_path_many)))
    EIGVECS, EIGVECS_0K, EIGVECS_MD = [], [], []
    for i in range(len(ks_path_many)):
        k = ks_path_many[i]
        k_scal = ks_scal_path_many[i]
        print('\t processing path k-point:', i, ks_scal_path_many[i])
        # NAC
        if NAC:
            if(np.allclose(k, [0,0,0])):
                kpoint_nac = ks_path_many[i+1]
            else:
                kpoint_nac = k
            eps_inf, BORN, Vuc = util.get_nac_params(BORN_file, SCell, N1, N2, N3, tot_atoms_uc)
            C_nac = util.get_nac_C(kpoint_nac, Vuc, BORN, eps_inf, masses_uc, N1N2N3)
            K_corr_0K, K_corr_renorm = util.get_K_corr(K, C_nac), util.get_K_corr(K_renorm, C_nac)
        else:
            K_corr_0K, K_corr_renorm = K, K_renorm
            
        # Renormalised          
        D_renorm = util.get_D(k, K_corr_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_renorm, eigvecs_renorm, omegas_renorm = util.get_eigvals_eigvec(D_renorm)
        EIGVECS.append(eigvecs_renorm)
        if(np.any(np.all(np.isclose(k_scal,ks_scaled_path), axis=1))):
            EIGVECS_MD.append(eigvecs_renorm)
           
        # 0K dispersion
        D_0K = util.get_D(k, K_corr_0K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_0K, eigvecs_0K, omegas_0K = util.get_eigvals_eigvec(D_0K)
        EIGVECS_0K.append(eigvecs_0K)
        frequencies[:,:,count_k] = omegas_renorm
        frequencies_disp[:,:,count_k] = omegas_0K 
        count_k = count_k + 1
# =============================================================================


# =============================================================================
    # this is for MD frequencies
    count_comm_k = 0
    frequencies_MD = np.zeros((2,tot_atoms_uc*3,len(ks_path)))
    ZQS = np.zeros((meta, 1+len(ks_path_many))) #for the DOS
    accepted_qpoints = []
    for i in range(len(ks_scaled_path)):
        qpoint = ks_scaled_path[i]
        if(np.any(np.all(np.equal(qpoint,ks_scaled_path[i]))) ): # e' un delirio questo ahaha, verifica che il qpoint trovato sia nel kpath
            to_skip = (4)*i
            to_skip_DOS = (1+meta)*i
            qpoint = np.genfromtxt('quasiparticles',skip_header=to_skip+0, max_rows=1)
            accepted_qpoints.append(qpoint.tolist())
            params = np.genfromtxt('quasiparticles',skip_header=to_skip+1, max_rows=3)
            omegas_gammas = params[0::2,:]
            frequencies_MD[:,:,count_comm_k] = omegas_gammas 
            if(np.allclose(qpoint, [0,0,0]) and NAC):
                # Renormalised          
                D_renorm_Gamma = util.get_D(qpoint, K_corr_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
                eigvals_renorm_Gamma, omegas_renorm_Gamma = util.get_eigvals(D_renorm_Gamma)
                frequencies_MD[0,:,count_comm_k] = omegas_renorm_Gamma
            count_comm_k = count_comm_k + 1
# =============================================================================

# =============================================================================
    # this is for the MD DOS
    ZQS = np.loadtxt('ZS', usecols=(0,1))
    if(gaussian_smoothing != 0):
        if(gaussian_smoothing < 0):
            print('automatic sigma not allowed in this mode, sigma set to 0.1')
            sig, mi = 0.1, freq[int(len(freq)/2)]
        else:
            sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
    
    gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2)                
    convolution = signal.fftconvolve(ZQS[:,1],gaussian, mode='same') / sum( gaussian)
    ZQS[:,1] = convolution
# =============================================================================
   
    # this is to reshape spectra according to real branches
    indexes, freqs_reshaped = util.get_real_branches(frequencies[0,:,:], EIGVECS, masses_uc, ks_path_many)
    indexes, freqs_disp_reshaped = util.get_real_branches(frequencies_disp[0,:,:], EIGVECS_0K, masses_uc, ks_path_many)
    indexes, freqs_MD_reshaped = util.get_real_branches(frequencies_MD[0,:,:], EIGVECS_MD, masses_uc, ks_path)
    frequencies_reshaped, frequencies_disp_reshaped , frequencies_MD_reshaped = np.copy(frequencies), np.copy(frequencies_disp), np.copy(frequencies_MD)
    frequencies_reshaped[0,:,:], frequencies_disp_reshaped[0,:,:], frequencies_MD_reshaped[0,:,:] = freqs_reshaped, freqs_disp_reshaped, freqs_MD_reshaped

    plot.plot40(distances_comm,  distances, frequencies_MD_reshaped, frequencies_disp_reshaped, frequencies_reshaped, ks_scaled_path, x_labels, ZQS, DOS_info, interp)

    
if(plot_type==5):
    print('Generating plot 5...')
    # take the path the user inputs and find all the k points you can plot, with interpolation
    ks_scal_path_many, ks_path_many, distances_many = util.get_kpath_many(ks_scaled_path, Hk, npoints=interp+1)
    num_modes = len(modes)
    K_renorm = np.loadtxt('K_renorm')

    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    
# =============================================================================        
    count_k = 0
    frequencies = np.zeros((tot_atoms_uc*3,len(ks_path_many)))
    frequencies_disp = np.zeros((tot_atoms_uc*3,len(ks_path_many)))
    EIGVECS, EIGVECS_0K = [], []
    for i in range(len(ks_path_many)):
        k = ks_path_many[i]
        print('\t processing path k-point:', i, ks_scal_path_many[i])
        
        # NAC
        if NAC:
            if(np.allclose(k, [0,0,0])):
                kpoint_nac = ks_path_many[i+1]
            else:
                kpoint_nac = k
            eps_inf, BORN, Vuc = util.get_nac_params(BORN_file, SCell, N1, N2, N3, tot_atoms_uc)
            C_nac = util.get_nac_C(kpoint_nac, Vuc, BORN, eps_inf, masses_uc, N1N2N3)
            K_corr_0K, K_corr_renorm = util.get_K_corr(K, C_nac), util.get_K_corr(K_renorm, C_nac)
        else:
            K_corr_0K, K_corr_renorm = K, K_renorm
            
        # Renormalised          
        D_renorm = util.get_D(k, K_corr_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_renorm, eigvecs_renorm, omegas_renorm = util.get_eigvals_eigvec(D_renorm)
        EIGVECS.append(eigvecs_renorm)
        
        # 0K dispersion
        D_0K = util.get_D(k, K_corr_0K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_0K, eigvecs_0K, omegas_0K = util.get_eigvals_eigvec(D_0K)
        EIGVECS_0K.append(eigvecs_0K)
        
        frequencies[:,count_k] = omegas_renorm
        frequencies_disp[:,count_k] = omegas_0K 
        count_k = count_k + 1
# =============================================================================

# =============================================================================      
    count = 0
    ZQS = np.zeros((meta, len(ks_scaled_path)))
    ZQS_proj = np.zeros((meta, tot_atoms_uc*3, len(ks_scaled_path)))
    accepted_qpoints = []
    for i in range(Nqs_path):
        to_skip = (1+meta)*i
        to_skip2 = 4*i
        Params = np.genfromtxt('quasiparticles',skip_header=to_skip2+1, max_rows=3)
        qpoint = np.genfromtxt('Zqs',skip_header=to_skip+0, max_rows=1)
        for j in range(len(ks_scaled_path)):
            if(np.allclose(qpoint, ks_scaled_path[j]) and ((qpoint.tolist() not in accepted_qpoints) or i==j)): #è un completo delirio
                accepted_qpoints.append(qpoint.tolist())
                
                Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
                ZQS[:,count] = Zq[:,0]
                for b in range(tot_atoms_uc*3):
                    params = Params[:,b]

                    if(gaussian_smoothing != 0):
                        if(gaussian_smoothing < 0):
                            sig, mi = params[-1]/2, freq[int(len(freq)/2)]
                            if (sig == 0):
                                sig = 0.0001
                        else:
                            sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
                        
                        gaussian = util.norm_gaussian(freq, mi, sig)
                        convolution = signal.fftconvolve(Zq[:,b+1],gaussian, mode='same') / sum( gaussian)
                        ZQS_proj[:,b,i] = convolution
                    else:
                        ZQS_proj[:,b,i] = Zq[:,b+1]
        count = count + 1
# =============================================================================      


    #this is to reshape spectra according to real branches
    indexes, freqs_reshaped = util.get_real_branches(frequencies, EIGVECS, masses_uc, ks_path_many)
    indexes_0K, freqs_disp_reshaped = util.get_real_branches(frequencies_disp, EIGVECS_0K, masses_uc, ks_path_many)
    frequencies_reshaped, frequencies_disp_reshaped  = np.copy(frequencies), np.copy(frequencies_disp)
    frequencies_reshaped, frequencies_disp_reshaped = freqs_reshaped, freqs_disp_reshaped
    
    ZQS_proj_reshaped = np.zeros((meta, num_modes, len(ks_path)))
    for j in range(len(ks_path)):
        ZQS_proj_reshaped[:,:,j] = ZQS_proj[:,indexes[j,modes],j]
    freqs_from_spectrum = util.freqs_spectrum(freq,ZQS_proj_reshaped)
    
    if (interp>0):
        ZQS_proj_interp = util.interpol_spectrum(ks_path, freq, freqs_from_spectrum, distances_comm, distances_many, frequencies_reshaped[modes,:], EIGVECS, modes, ZQS_proj_reshaped)
    else:
        ZQS_proj_interp = ZQS_proj_reshaped[:,range(len(modes)),:]
        
    plot.plot_k2(distances_comm, distances_many, frequencies_disp_reshaped, frequencies_reshaped, freq,ZQS,ZQS_proj_interp, x_labels,max_Z,modes)


if(plot_type==6):
    num_modes = len(modes)
    # ks_scal_path_many, ks_path_many, distances_many = util.get_kpath_many(ks_scaled_path, Hk, npoints=interp+1)
    K_renorm = np.loadtxt('K_renorm')
    
    freq = np.loadtxt('ZS', usecols=0)
    meta = len(freq)
    accepted_qpoints = []
    
    for i in range(Nqs_path):
        k = ks_path[i]
        #NAC
        if (NAC and Nqs_path>1):
            if(np.allclose(k, [0,0,0])):
                kpoint_nac = ks_path[i+1]
            else:
                kpoint_nac = k
            eps_inf, BORN, Vuc = util.get_nac_params(BORN_file, SCell, N1, N2, N3, tot_atoms_uc)
            C_nac = util.get_nac_C(kpoint_nac, Vuc, BORN, eps_inf, masses_uc, N1N2N3)
            K_corr_0K, K_corr_renorm = util.get_K_corr(K, C_nac), util.get_K_corr(K_renorm, C_nac)
        else:
            K_corr_0K, K_corr_renorm = K, K_renorm
        
        D_0K = util.get_D(k, K_corr_0K,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_0K, eigvecs_0K, omegas_0K = util.get_eigvals_eigvec(D_0K)
        
        D_renorm = util.get_D(k, K_corr_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_renorm, eigvecs_renorm, omegas_renorm = util.get_eigvals_eigvec(D_renorm)
        
        to_skip = (1+meta)*i
        Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
        for b in range(len(modes)):
            mm = modes[b]
            ani = plot.plot6(freq,Zq[:,mm+1],Zq[:,0], k,eigvecs_0K[:,mm],omegas_0K[mm],mm,Ruc,masses_uc, max_Z, title='0K eigenvectors')
        for b in range(len(modes)):
            mm = modes[b]
            ani = plot.plot6(freq,Zq[:,mm+1],Zq[:,0], k,eigvecs_renorm[:,mm],omegas_renorm[mm],mm,Ruc,masses_uc, max_Z, title='finite T eigenvectors')
    
    
if(plot_type==7):
    print('WARNING: atm this only works for cubic crystals!')
    
    K_renorm = np.loadtxt('K_renorm')
    # take the path the user inputs and find all the k points you can plot
    # ks_path, x_labels, distances = read.read_path(kinput, Nqs_path, labels, Hk) 
    
    c_A_to_B = 1.88973 #conversion to Bohrs
    Hk = Hk/(2*np.pi) #if you use Thz as frequencies, you have to use 1/B as k, without 2 pi
    Hk = Hk*c_A_to_B #if you wanna use the 1/100 conv factor later, you need Angstrom
    
    #find the closest qpoints for EC
    dirs_C11 = [[1,0,0],[0,1,0],[0,0,1]]
    dirs_C12_C44 = [[1,1,0],[1,0,1],[0,1,1]]
    ampl = 10
    
    A1, A2, A3 = SCell[0,:], SCell[1,:], SCell[2,:]
    V = np.dot(A1, np.cross(A2,A3))
    print('The volume of the supercell is ', V, 'Bohr^3')
    print('NB: if you run a NPE simulation, you better use the average NPE volume!')
    conv_to_gcm3 = 11.2059 #convert amu/B^3 to g/cm^3
    M_tot = np.sum(masses)/3
    rho = M_tot/V*conv_to_gcm3
    print('The density is ',rho, ' g/cm^3\n')
    
    conv_factor = 1/100        
    
    accepted_qpoints = []
    for i in range(1,len(ks_path)-1):
        
        
        qpoint_sc = ks_scaled_path[i]
        qpoint_sc_prev = ks_scaled_path[i-1]
        k = ks_path[i]
        # Renormalised          
        D_renorm = util.get_D(k, K_renorm,N1,N2,N3,R0,masses_uc, tot_atoms_uc, SCell)
        eigvals_renorm, eigvec_q, omegas_renorm = util.get_eigvals_eigvec(D_renorm)
        
        
        direc = np.abs(np.dot(Hk, qpoint_sc - qpoint_sc_prev))
        direc_sc = np.round(direc/direc.max(),2)

            
        to_skip = (4)*i
        qpoint = np.genfromtxt('quasiparticles',skip_header=to_skip+0, max_rows=1)
        if(direc_sc.tolist() in dirs_C11 and (np.allclose(qpoint_sc_prev,[0,0,0]) or np.allclose(ks_path[i+1],[0,0,0]))):
            
            params = np.genfromtxt('quasiparticles',skip_header=to_skip+1, max_rows=3)
            omegas  = params[0,:]
            
            csi = np.linalg.norm(np.dot(Hk,qpoint))
            print('csi: ', csi)
            print()
            ##==============================================================================
            ## C11
            print('# =============================================================================')
            print('C11')
            freqs = []
            eigvecs_acou = eigvec_q[:,0:3]
            for j in range(0,3):
                freqs.append(omegas[j])
                
            
            j = -1
            freq = freqs[j]
            eigvec = eigvecs_acou[:,j]
            print('qpoint scaled: ', qpoint)
            print('qpoint:', np.round(np.dot(Hk,qpoint),3))
            print()
            print('freqs: ', np.round(freqs,3), ' taken ', np.round(freq,3))
            print('\neigenvec:')
            print(np.round(eigvec*ampl/np.sqrt(masses_uc),2).reshape(np.shape(Ruc)))
            print('# =============================================================================')
            print()
            C11 = rho * (freq/csi)**2 *conv_factor
            ##==============================================================================
            
            # =============================================================================
            # C44
            print('# =============================================================================')
            print('C44-1')
            freqs = []
            eigvecs_acou = eigvec_q[:,0:3]
            for j in range(0,3):
                freqs.append(omegas[j])
            j = 0
            freq = freqs[j]
            eigvec = eigvecs_acou[:,j]
            print('qpoint scaled: ', qpoint)
            print('qpoint: ',np.round(np.dot(Hk,qpoint),3))
            print()
            print(np.round(freqs,3), ' taken ', np.round(freq,3))
            print('\neigvec:')
            print(np.round(eigvec*ampl/np.sqrt(masses_uc),2))
            C44_1 = rho * (freq/csi)**2 *conv_factor
            print('# =============================================================================')
            print()
            # =============================================================================
            
            
        if(direc_sc.tolist() in dirs_C12_C44 and np.allclose(qpoint_sc_prev,[0,0,0]) or np.allclose(ks_path[i+1],[0,0,0])):
            params = np.genfromtxt('quasiparticles',skip_header=to_skip+1, max_rows=3)
            omegas  = params[0,:]
            
            #check that k = [csi, csi, 0]
            check = [np.allclose(x,csi) for x in np.dot(Hk,qpoint)] 
            if (check not in [[True, True, False],[True, False, True],[False, True, True]]):
                print('ERROR WARNING: the q point is not [csi, csi, 0] so everything is gonna be wrong')
            
            
            
            # =============================================================================
            # C44
            print('# =============================================================================')
            print('C44-2')
            freqs = []
            eigvecs_acou = eigvec_q[:,0:3]
            for j in range(0,3):
                freqs.append(omegas[j])
            j = 1
            freq = freqs[j]
            eigvec = eigvecs_acou[:,j]
            print('qpoint scaled: ', qpoint)
            print('qpoint:', np.dot(Hk,qpoint))
            print()
            print('freqs: ', freqs, ' taken ', freq)
            print('\neigenvec:')
            print(np.round(eigvec*ampl/np.sqrt(masses_uc),2).reshape(np.shape(Ruc)))
            print('# =============================================================================')
            print()
            C44_2 = .5* rho * (freq/csi)**2 *conv_factor
            # =============================================================
            
            
            #==============================================================================
            # C12
            print('# =============================================================================')
            print('C12-1')
            freqs = []
            eigvecs_acou = eigvec_q[:,0:3]
            for j in range(0,3):
                freqs.append(omegas[j])
                
            #csi = np.linalg.norm(np.dot(hr,qpoint_scaled))
            j = 0
            freq = freqs[j]
            eigvec = eigvecs_acou[:,j]
            print('qpoint scaled: ', qpoint)
            print('qpoint: ',np.round(np.dot(Hk,qpoint),3))
            print()
            print(np.round(freqs,3), ' taken ', np.round(freq,3))
            print('\neigvec:')
            print(np.round(eigvec*ampl/np.sqrt(masses_uc),2))
            A = rho * (freq/csi)**2 *conv_factor
            C12_1 =C11-A  #<- with freqs[0] # A - C11 -2*C44 #
            print('# =============================================================================')
            print()
            #==============================================================================
            
            #==============================================================================
            # C12
            print('# =============================================================================')
            print('C12-2')
            freqs = []
            eigvecs_acou = eigvec_q[:,0:3]
            for j in range(0,3):
                freqs.append(omegas[j])
                
            #csi = np.linalg.norm(np.dot(hr,qpoint_scaled))
            j = -1
            freq = freqs[j]
            eigvec = eigvecs_acou[:,j]
            print('qpoint: ', qpoint)
            print('qpoint:', np.dot(Hk,qpoint))
            print()
            print('freqs: ', freqs, ' taken ', freq)
            print('eigenvec:')
            print(np.round(eigvec*ampl/np.sqrt(masses_uc),2).reshape(np.shape(Ruc)))
            print('# =============================================================================')
            print()
            A = rho * (freq/csi)**2 *conv_factor
            C12_2 = A - C11 -2*C44_1 #C11-A  #<- with freqs[0] #
            C12_2_2 = A - C11 -2*C44_2
            #==============================================================================
            
            print()
            print('Elastic constants')
            print('C11 = ', np.round(C11,2), 'GPa')
            print('C44 using [100] = ', np.round(C44_1,2), 'GPa')
            print('C44 using [110] = ', np.round(C44_2,2), 'GPa')
            print('C12 using eq 1 = ', np.round(C12_1,2), 'GPa')
            print('C12 using eq 2 (C44[100])= ', np.round(C12_2,2), 'GPa')
            print('C12 using eq 2 (C44[110])= ', np.round(C12_2_2,2), 'GPa')
            
            print()
            print('Engineering elastic constants')
            ni = C12_2_2/(C11+C12_2_2)
            E = C11*(1+ni)*(1-2*ni)/(1-ni)
            print('E = ', np.round(E,2), 'GPa')
            print('ni = ', np.round(ni,2))
            print('G = ', np.round(C44_2,2), 'GPa')
    
    

        
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 13 14:12:07 2019

@author: gc4217
"""
import numpy as np
import os, sys

def avg_vs_t(time,quantity):
    avg = []
    for i in range(1,len(time)):
        dt = time[i]-time[i-1]
        y = quantity[0:i]
        integral = np.sum(dt*y,axis=0)
        avg.append(1/(i*dt)*integral)
    avg = np.array(avg)
    return avg


N1, N2, N3 = int(sys.argv[1]), int(sys.argv[2]), int(sys.argv[3])
n_uc = int(sys.argv[4])
N1N2N3 = N1*N2*N3
N = N1*N2*N3*n_uc
DT = float(sys.argv[5])
flag  = sys.argv[6]

if(flag=='ASAP' or flag=='asap'):
    print('Hello, Im now converting the ASAP output files...\n')
    
    with open('Cartesian_0','r') as f:
        A = f.readlines()
    a = 0
    for i in range(0,len(A),N+1):
        del A[i-a]
        a = a + 1
        
    with open('pos','w') as g:
        g.writelines(A)
        
    Rall = np.loadtxt('pos',usecols=(1,2,3))
    
    Num_timesteps = int(len(Rall)/N)
    t = np.arange(Num_timesteps-1)*DT*2.418884254*1e-05 #conversion to picoseconds
    dt_ps = t[1]-t[0]
    
    Rt = []
    for i in range(0,len(Rall),N):
        if(i==N):
            continue
        #pos_first_Ba = Rall[i:i+1,:]
        #pos_cm = np.sum(Rall[i:i+N,:],axis=0)/N
        
        to_write = Rall[i:i+N:1] #- pos_cm
        Rt.append(to_write.flatten())
        
    Rt = np.array(Rt)
    #Vt = np.gradient(Rt,dt_ps,axis=0)  
    Vt = np.diff(Rt,axis=0)/dt_ps
    
    data_pos = np.hstack((t.reshape(len(t),1),Rt))
    runn_avg = avg_vs_t(t,Rt)
    data_avg = np.hstack((t[1::].reshape(len(t[1::]),1),runn_avg))
    
    np.savetxt('runn_avg_pos',data_avg)
    np.savetxt('data_pos',data_pos)
    
    
    data_vel = np.hstack((t[0:-1].reshape(len(t[0:-1]),1),Vt))
    runn_avg = avg_vs_t(t,Vt)
    data_avg = np.hstack((t[0:-1].reshape(len(t[0:-1]),1),runn_avg))
    
    np.savetxt('runn_avg_vel',data_avg)
    #np.savetxt('data_vel',data_vel)
    
    
    os.remove('pos')
    
    print('Finished. data_pos, data_vel, runn_avg_pos and runn_avg_vel created.\n\n')
    print('Please copy the file data_pos to your phonon DOS folder!')


if(flag=='QE' or flag=='qe'):
    name = sys.argv[6]
    
    print('Hello, Im now converting the QE output files...\n')
    
    with open(name+'.pos','r') as f:
        A = f.readlines()
    a = 0
    for i in range(0,len(A),N+1):
        del A[i-a]
        a = a + 1
        
    with open('pos','w') as g:
        g.writelines(A)
        
    Rall = np.loadtxt('pos')
    
    Num_timesteps = int(len(Rall)/N)
    t = np.arange(Num_timesteps-1)*DT*2.418884254*1e-05 #conversion to picoseconds
    
    R = []
    for i in range(0,len(Rall),N):
        if(i==N):
            continue
        #pos_first_Ba = Rall[i:i+1,:]
        #pos_cm = np.sum(Rall[i:i+N,:],axis=0)/N
        
        to_write = Rall[i:i+N:1] #- pos_cm
        R.append(to_write.flatten())
        
    R = np.array(R)
    
    data_pos = np.hstack((t.reshape(len(t),1),R))
    
    runn_avg = avg_vs_t(t,R)
    data_avg = np.hstack((t[1::].reshape(len(t[1::]),1),runn_avg))
    
    np.savetxt('runn_avg_pos',data_avg)
    np.savetxt('data_pos',data_pos)
    
    with open(name+'.vel','r') as f:
        A = f.readlines()
    a = 0
    for i in range(0,len(A),N+1):
        del A[i-a]
        a = a + 1
        
    with open('vel','w') as g:
        g.writelines(A)
        
    Vall = np.loadtxt('vel')
    
    Num_timesteps = int(len(Vall)/N)
    t = np.arange(Num_timesteps-1)*DT*2.418884254*1e-05 #conversion to picoseconds
    
    V = []
    for i in range(0,len(Vall),N):
        if(i==N):
            continue
        #pos_first_Ba = Rall[i:i+1,:]
        #pos_cm = np.sum(Rall[i:i+N,:],axis=0)/N
        
        to_write = Vall[i:i+N:1] #- pos_cm
        V.append(to_write.flatten())
        
    V = np.array(V)
    
    data_vel = np.hstack((t.reshape(len(t),1),V))
    
    runn_avg = avg_vs_t(t,V)
    data_avg = np.hstack((t[1::].reshape(len(t[1::]),1),runn_avg))
    
    np.savetxt('runn_avg_vel',data_avg)
    np.savetxt('data_vel',data_vel)
    
    
    os.remove('vel')
    
    
    
    print('Finished. data_pos, data_vel, runn_avg_pos and runn_avg_vel created.\n\n')
    print('Please copy the file data_pos to your phonon DOS folder!')






















